\documentclass[a4paper]{article}
\usepackage{calc}
%\usepackage[draft]{graphicx}
\usepackage{graphicx}
\usepackage{lambdaTeX-igl}
\usepackage{Haskell2LaTeX}
%\usepackage{hyperref}
\usepackage{url}
\usepackage[sort]{cite}
\usepackage{amsmath}
\usepackage{subfigure}
\usepackage{example}
\usepackage{alltt}
\usepackage{lscape}
\usepackage{bbding}
\font\crestfont=newcrest

% Adapted from tex/plain/base/webmac.tex
\font\tentex=cmtex10
\def\WEB{{\tentex WEB}}
\def\TANGLE{{\tentex TANGLE}}
\def\WEAVE{{\tentex WEAVE}}

\def\hkeyword#1{\hbox{\textbf{#1}}}
\def\hvarid#1{\hbox{\textit{#1}}}
\def\htycon#1{\hbox{\textsf{#1}}}
\def\filename#1{\hbox{\texttt{#1}}}
\def\program#1{\hbox{\texttt{#1}}}
\def\environment#1{\hbox{\texttt{#1}}}
\def\latexpackage#1{\hbox{\texttt{#1}}}

\newenvironment{iglist}
    {\begin{list}{}%
        {\renewcommand{\makelabel}[1]{\emph{##1}}%
         \setlength{\leftmargin}{3em}}}%
    {\end{list}}

\begin{document}

\begin{titlepage}
\null\vfil
\begin{center}
\lineskip .75em
{\Huge \HaskellToLaTeX\\\LARGE A Haskell pretty-printer}
\vskip 3em
{\large Candidate number: 13243}
\vskip 3em
{\large Computation undergraduate project\\
Trinity Term, 2002}
\vskip 3em
\crestfont\char0
\vfil\null
\end{center}
\end{titlepage}

\begin{center}
\lineskip .75em
{\LARGE \HaskellToLaTeX\\A Haskell pretty-printer}
\vskip 1em
{\large Candidate number: 13243}
\vskip .75em
{\large Computation undergraduate project\\
Trinity Term, 2002}
\end{center}
\vskip 1em
\renewcommand{\abstractname}{Summary}
\begin{abstract}
Haskell\footnote{Throughout this report the term ``Haskell'' will be
used to refer to Haskell~98~\cite{haskell98report}.} allows a
script to be written in a literate style. It is designed such that a
script written in this style can be written in a {\TeX} style with the
executable code surrounded by \verb!\begin{code}! and \verb!\end{code}!.

While neither plain {\TeX} nor {\LaTeX} have a \environment{code} environment it
is trivial to define a simple environment to show the code. For example,
in {\LaTeX}, one can make it an alias for the verbatim package with
\begin{verbatim}
\usepackage{verbatim}
\newenvironment{code}{\verbatim}{\endverbatim}
\end{verbatim}

However, researchers publishing documents containing Haskell code do not
tend to print the code verbatim, but prefer to make use of the
typesetting features available such as printing keywords in
\textbf{bold}, \verb!->! as $\rightarrow$ and the type variable \verb!a!
as $\alpha$. At the time this project was started there were three tools
for typesetting Haskell in {\TeX} or {\LaTeX}
available~\cite{haskellstyle,lhs2tex,haskellsty} and towards the
completion of this project a fourth was released~\cite{lambdatex}.
However, as we will see, all four have their shortcomings.

Buried deep in the GHC source code we find the following words of
wisdom:
\begin{quote}
It is hoped that this style of programming will
encourage the writing of accurate and clearly documented programs
in which the writer may include motivating arguments, examples
and explanations.\\
\hspace*{4em}---
\vtop{fptools/ghc/utils/unlit/unlit.c\\
      in the Haskell CVS repository~\cite{haskellcvs}}
\end{quote}
However, few, if any, such scripts exist. Perhaps part of the reason is
the difficulty and lack of automation in turning such a script into a
nicely typeset document. It is the aim of this project to produce a
Haskell program which, when run on a literate Haskell script, will
produce a {\LaTeX} document that is the same as the original but with
the code sections typeset nicely.
\end{abstract}
\clearpage
\tableofcontents
\clearpage
\listoffigures
\clearpage
\section{Introduction}

\subsection{Literate programming}

Knuth~\cite{knuth} is generally regarded as the founding father of
literate programming~\cite{literate_programming}; his paper with this
name~\cite{literate_programming_paper} in 1984 introduced the concept to
the world and his book of the same name~\cite{literate_programming_book}
in 1992 expands on the ideas. In his original paper he tells us:

\begin{quote}
I believe that the time is ripe for significantly better documentation
of programs, and that we can best achieve this by considering programs
to be works of literature. Hence, my title: ``Literate Programming.''

Let us change our traditional attitude to the construction of programs:
Instead of imagining that our main task is to instruct a computer what
to do, let us concentrate rather on explaining to human beings what we
want a computer to do.
\end{quote}

Among the literate programming systems that exist today is Knuth's
\WEB~system~\cite{web}, introduced by his paper, in which a literate
script is composed of multiple sections, each of which can have {\TeX}
documentation, macro definitions and Pascal code. The utility {\WEAVE}
can then be used to produce a {\TeX} document comprised of the {\TeX}
documentation in the script along with the macros and Pascal code
pretty-printed, while its companion tool {\TANGLE}, when given the same
{WEB} script, takes the code part of each section and arranges the
pieces in the correct order to create a correct Pascal program.

Numerous other literate programming systems exist for a large number of
programming language and documentation language combinations. The
Haskell report defines a style of writing Haskell scripts which provides
many, but not all\footnote{Notably it does not allow functions to be
written in out of order fragments. This is less important in a language
like Haskell where there are no variable declarations and values can be
bound to variables out of order.}, of the features we would expect of a
literate programming system. A significant difference to the {\WEB}
system is that Haskell compilers and interpreters work directly on the
literate script.

Once a format in which literate scripts can be written has been defined
two things are needed to complete the literate programming system.
Firstly it must be possible to take a literate script and either
interpret it or compile it to executable code. This by itself would only
constitute a conventional programming system; for a literate programming
system it must also be possible to take a literate script and produce
the document described by the script. Haskell is fortunate enough to
have three separate, high quality implementations of the former but is
sadly lacking in the latter. We aim here to redress the balance.

\subsection{This project}

We will create a tool which takes a literate script and extracts the code
fragments like:

\begin{samepage}
\begin{verbatim}
\begin{code}
qsort :: [a] -> [a]
qsort []     = []
qsort (x:xs) = qsort xs_lt_x ++ [x] ++ qsort xs_greq_x
    where xs_lt_x   = [y | y <- xs, y < x]
          xs_greq_x = [y | y <- xs, y >= x]
\end{code}
\end{verbatim}
\end{samepage}

\noindent These fragments are then parsed and LaTeX markup for the
pretty-printed code is substituted, giving a result like this:

\begin{code}
qsort :: [a] -> [a]
qsort []     = []
qsort (x:xs) = qsort xs_lt_x ++ [x] ++ qsort xs_greq_x
    where xs_lt_x   = [y | y <- xs, y < x]
          xs_greq_x = [y | y <- xs, y >= x]
\end{code}

\noindent All of the code in this report, including the \hvarid{qsort}
example immediately above, has been typeset using the program developed.

This report continues with a discussion of relevant existing theory and
technologies, including a look at existing Haskell pretty-printers, in
Section~\ref{sec:existing_theory}. This is followed by the writeup of
the bulk of the work, the parser, in Section~\ref{sec:parsing}---this is
largely the development of a parser combinator library, described in
Section~\ref{sec:PCs}. The remainder of the work, the output phase,
follows in Section~\ref{sec:pp}. We conclude with a discussion of the
value of the work and possible future developments in
Section~\ref{sec:discussion}.

This project makes a number of contributions back to the community.
These include a flexible parser combinator library, a parser for Haskell
built upon this library and a pretty-printer for Haskell built upon this
parser.

\section{Existing theory and technologies}\label{sec:existing_theory}

\subsection{Haskell}

Haskell~\cite{haskell} is a polymorphically typed, lazy, purely
functional language. The language specification allows a style of script
that encourages the typesetting of both code and description in a {\TeX}
document. This is both the language which we will pretty-print and the
language in which we write the pretty-printer. We choose to write the
pretty-printer in Haskell partly due to the strength of the language in
parsing and tree manipulation and partly because we believe the
basic tools for any self-respecting language should be written in that
language.

An introduction to the Haskell language is beyond the scope of this
report. Readers unfamiliar with the language are encouraged to read one
of the books available on the subject~\cite{IFP,craft_of_fp}.

\subsection{Parsing}\label{sec:theory_parsing}

The art and science of parsing has been of interest to computer
scientists for decades and much of the current theory and practices
in use today date from the 70s and 80s. Indeed the canonical
reference~\cite{dragonbook} was published in 1986.

The normal overall strategy is to first \emph{lexically
analyse}\footnote{\emph{lexically analysing} is commonly shortened to
\emph{lexing} and also known as \emph{scanning}.} the input to produce a
list of tokens from the list of characters, normally dropping the
whitespace and comments, and then \emph{syntactically
analyse}\footnote{\emph{syntactical analysing} is also commonly refered
to as \emph{parsing}, but we will avoid this usage, instead referring to
the process of lexical analysis followed by syntax analysis as parsing.}
the list of tokens to get an abstract syntax tree.

The two most common strategies employed for syntax analysis are called
top-down and bottom-up. Bottom-up SAs start from the terminal symbols
and build up to progressively larger language constructs; typical
examples are so-called \emph{shift-reduce parsers} which generally
consist primarily of a large table describing a state machine. As such
they tend to be generated mechanically from an abstract description of
the grammar. Top-down SAs, i.e., starting with the root non-terminal and
substituting non-terminals for one of their productions in a depth-first
fashion, are much easier to write and check by hand but efficient
implementations can only be created for a more restrictive set of
grammars.

\subsection{Parsing with parser combinators}\label{sec:parser_combinators}

Parser Combinators, or PCs, provide a way of building lexical analysers
and top-down syntax analysers by combining smaller parsers as the name
suggests. Typically you will be provided with (at least):

\begin{iglist}
\item[Fail]{
    A parser that consumes none of a list of items of type $\alpha$ and
    fails.
}
\item[Succeed]{
    A function that takes a value $x$ of type $\beta$ and returns a parser
    that consumes none of a list of items of type $\alpha$ and
    successfully returns $x$.
}
\item[Match]{
    A function that takes a value $x$ of type $\alpha$ and returns a
    parser that takes a list of items of type $\alpha$ and either
    successfully returns $x$ and removes the first item of input from
    the list if the first item in the list is equal to $x$ or fails
    otherwise.
}
\item[Sequential Composition]{
    A function that, given two parsers, applies the first parser and, if
    it succeeds, applies the second to the input as it is after the
    first parser has been applied. If either of the parsers fails then
    the sequential composition of the two fails. If both are successful
    then what happens depends somewhat on the implementation. In the
    Haskell world where we are blessed with functions as first class
    values it is common for the result to be the value returned by the
    first parser applied to the value returned by the second parser.
}
\item[Choice]{
    A function that, given two parsers, decides in some way whether to
    return the result of the first or the second. Both parsers are tried
    on a copy of the original input stream and the new input stream is
    as it was after the chosen parser was applied to it. Common rules
    for determining which to chose are:
    \begin{itemize}
    \item{
        If the first parser successfully consumes any input or the
        second parser fails then return the result of the first parser.
        Otherwise return the result of the second parser, which could be
        a failure if both parsers fail.
    }
    \item{
        If both parsers succeed then return the result of the one that
        consumes the most input, or the first if they consume the same
        amount of input as each other. If only one succeeds then the
        result of that parser is returned, otherwise failure is
        returned.
    }
    \end{itemize}
}
\end{iglist}

Two general purpose implementations of parser combinator libraries for
Haskell exist~\cite{uuparsingcombinators,parsec} as well as a document
explaining how monadic parser combinators can be implemented in a
Haskell-like language~\cite{MPCs}.

\subsection{Typesetting in (plain) {\TeX}}

Typesetting, ``the act or art of setting type''~\cite{web1913}, dates
back to before programmable computers even existed when printing
companies typeset documents by hand to be printed on printing machines.
As computers appeared, so too did computer based typesetting systems,
although these were hardware and OS specific.

When Knuth saw Addison-Wesley's typesetting of the 2nd edition of
volume~2 of ``The Art of Computer Programming'' he was appalled and so, in May
1977, started work on {\TeX}, which later he described as ``a new
typesetting system intended for the creation of beautiful books---and
especially for books that contain a lot of mathematics'' in the preface
of the {\TeX}book~\cite{texbook}. Two versions of {\TeX}
exist---{\TeX}78 and {\TeX}82. The former is now considered obsolete and
{\TeX} is taken to mean {\TeX}82 unless otherwise specified. A prototype
was started in the summer of 1977 followed by a complete implementation
from late 1977 to early 1978. Version 0 was released in September 1982,
and version 1 in November 1983. The current version, 3.14159, was
released in March 1995 and has had only minor corrections to the
documentation since---a remarkable stability record compared to other
complex pieces of software.

For more information on {\TeX} the definitive reference is the
{\TeX}book~\cite{texbook}, although the USENET group comp.text.tex and
\cite{ctan} are also useful. For more on the history of {\TeX}
see~\cite{errorlogtex,texweb} and try searching for ``history of TeX''
in comp.text.tex on Google~Groups~\cite{google}. This report will assume
the reader is familiar with the basics of using {\TeX} to typeset
documents.

{\TeX} itself has only the primitive commands defined. When people talk
about writing in {\TeX} they generally really mean plain
{\TeX}---although some of the more confused actually mean {\LaTeX}
which we will look at in the next section! A simple plain {\TeX}
document source and output is shown in Figure~\ref{fig:tex_input_output}.

\setbox0\hbox{\fbox{\begin{minipage}{\hsize}
\font\titlefont=cmr7 scaled\magstep4
\def\title#1{
    \vskip 15pt plus 3pt minus 3pt
    {\titlefont #1}
    \vskip 10pt plus 3pt minus 3pt
}

\title{Foo}

Hello world---here is quite a long paragraph with some {\bf bold text} in. {\TeX} will do the line wrapping wrap for us.

\title{Bar}

Some maths for you:
$x^{2y} + \int_0^\infty f\left(\sin(x)\over x\right)\,dx$
or, in display mode,

$$x^{2y} + \int_0^\infty f\left(\sin(x)\over x\right)\,dx$$
\end{minipage}}}

\begin{figure}
\subfigure[Input]{
\fbox{
    \begin{minipage}{\hsize}
    \verbatiminput{doc.tex.inc}
    \end{minipage}
}
\label{fig:tex_input}
}

\subfigure[Output]{
\box0
\label{fig:tex_output}
}
\caption{Example plain {\TeX} document}
\label{fig:tex_input_output}
\end{figure}

\subsection{\LaTeX}

The {\LaTeX} companion~\cite{latexcompanion} tells us ``{\LaTeX} is a
generic typesetting system that uses {\TeX} as its formatting engine.''.
It was written in the early 1980s by Leslie Lamport who released 2.09,
his final release, in April 1986. Then maintenance was taken over by the
{\LaTeX}3 Project~\cite{latex3} who are responsible for {\LaTeXe}, the
current release.

Among the additional features offered by {\LaTeX} are a package system
and many packages; we will find \latexpackage{tabularx}, \latexpackage{ifmtarg} and
\latexpackage{verbatim} particularly useful. For further information on
{\LaTeX}, see~\cite{latexguide,lshort,latexcompanion,latex3}. An example
of a simple {\LaTeX} document is given in
Figure~\ref{fig:latex_input_output}. As you can see, it is a higher level
language with a more structured appearance. This report will assume the
reader is familiar with the basics of using {\LaTeX} to typeset
documents.

\setbox0\hbox{\fbox{\begin{minipage}{\hsize}

\section*{Foo}

Hello world---here is quite a long paragraph with some
\textbf{bold text} in. {\LaTeX} will do the line wrapping wrap for us.

\section*{Bar}

Some maths for you:
$x^{2y} + \int_0^\infty f\left(\sin(x)\over x\right)\,dx$
or, in display mode,

\begin{displaymath}
x^{2y} + \int_0^\infty f\left(\sin(x)\over x\right)\,dx
\end{displaymath}
\end{minipage}}}

\begin{figure}
\subfigure[Input]{
\fbox{
    \begin{minipage}{\hsize}
    \verbatiminput{doc.latex.inc}
    \end{minipage}
}
\label{fig:latex_input}
}

\subfigure[Output]{
\box0
\label{fig:latex_output}
}
\caption{Example {\LaTeX} document}
\label{fig:latex_input_output}
\end{figure}

\subsection{Pretty-printing Haskell in {\TeX} and {\LaTeX}}

This is not the first attempt to produce a package that pretty-prints
Haskell scripts in {\TeX} or {\LaTeX}. The ``Libraries and Tools for
Haskell'' section of the Haskell webpage~\cite{haskell} has a subsection
``Typesetting Haskell in {\TeX}'' in which four packages are
listed~\cite{lambdatex,haskellstyle,lhs2tex,haskellsty}, one of which
was only released towards the end of this project.

\subsubsection{Haskell Style for LaTeX2e}

The first~\cite{haskellsty} is not intended to take a Haskell program
and typeset it. Instead it provides a set of commands with which a
Haskell program can be typeset by hand. Although these give the author
full control over the final layout it means that the code is no longer
executable.

\subsubsection{haskell.sty}

The next contender, \filename{haskell.sty}~\cite{haskellstyle}, is a simple
{\LaTeX} style file which uses the listings package to do the
pretty-printing. It makes no attempt to print keywords in bold and,
while it will correctly pretty-print \verb!=>! as $\Rightarrow$ it is
easily confused and will, for example, pretty-print \verb!<=>! as
$\le>$.

\subsubsection{\lambdaTeX}

\lambdatexfunctions{foo}

The way the author of {\lambdaTeX}~\cite{lambdatex} wrote his
\filename{Example.lhs} shows the package at its best, with the code being
written in a similar style to the code below. In particular note that
keywords starting blocks are on a line of their own and the first line
of the block is on the next line. Also note that each level of
indentation is a single space.

\begin{verbatim}
> foo =
>  do
>   putStrLn "Hello world!"
>   putStrLn "Goodbye!"
> foo = bar1 + bar2
>  where
>   bar1 = 2 * bar1'
>    where
>     bar1' = 3
>   bar2 = 2
\end{verbatim}
\clearpage
\noindent The typeset version is below. This looks like valid Haskell as
intended.

\csname hs@beginBird\endcsname foo =
   do
    putStrLn "Hello world!"
    putStrLn "Goodbye!"
  foo = bar1 + bar2
   where
    bar1 = 2 * bar1'
     where
      bar1' = 3
    bar2 = 2


However, suppose we now write some code in a style preferred by this
author.

\begin{verbatim}
> foo = bar1 + bar2
>   where bar1 = 2 * bar1'
>             where bar1' = 3
>         bar2 = 2
\end{verbatim}

When this is typeset by the package, not only is the second \hkeyword{where}
keyword indented far more than we would like, the code no longer looks
valid---the definition of \hvarid{bar2} is indented more than the
definition of \hvarid{bar1}.

\csname hs@beginBird\endcsname foo = bar1 + bar2
    where bar1 = 2 * bar1'
              where bar1' = 3
          bar2 = 2

This is a necessary limitation of any pretty-printer which works only on
individual tokens and does not use a monospaced font.

Also conspicuous by their absence in \filename{Example.lhs} are case
statements. Both styles of laying out case statements that this
author is familiar with are shown below.

\begin{verbatim}
> foo x = case x of
>             0 -> 0
>             _ -> 1
> foo x = case x of 0 -> 0
>                   _ -> 1
\end{verbatim}

As the typeset version below shows, the first of these has the caselets
further to the right than is aesthetically pleasing and the second
does not have the second caselet aligned with the first, as with the
\hkeyword{where} example above. The problem is of course the same limitation as
mentioned above.

\csname hs@beginBird\endcsname foo x = case x of
              0 -> 0
              _ -> 1
  foo x = case x of 0 -> 0
                    _ -> 1

\subsubsection{\program{lhs2tex}}

Perhaps the best thought out is \program{lhs2TeX}~\cite{lhs2tex}. It offers
two output modes, one using a monospaced font and the other a
proportional font. The monospaced font mode gets around the problems
with lexing only pretty-printers, but the result is less pleasing to the
eye. In the other mode an alignment column can be specified and
everything from that column onwards is aligned; for example, in the
example input in Figure~\ref{fig:lhs2tex_demo} you would set the
alignment column to 13 so the right hand sides were all aligned. This is
an interesting idea, although it would be better if it could be
specified more locally.

\begin{figure}
\begin{obeyspaces}
\begin{tabular}{l}
\texttt{foo         = x + y}\\
\texttt{    where x = 10}\\
\texttt{          y = 20}
\end{tabular}
\end{obeyspaces}
\caption{Example \program{lhs2TeX} input}
\label{fig:lhs2tex_demo}
\end{figure}

\subsubsection{Related programs}

There are a few other programs which do related tasks; we mention them
here for completeness. The filter \program{detex} removes {\LaTeX} and
{\TeX} control sequences; when a literate script is filtered through it
reasonable plain text documentation is output. The GNU \program{enscript}
implementation comes with a Haskell pretty-printing mode which can be
used to highlight Haskell scripts with either boldface or colour;
unfortunately it gets confused and goes wrong very easily. Syntax
highlighting modes for a number of editors exist and some also support
automatic indentation. There are also three Haskell source browsers:
HDoc, HaskellDoc and ``The Haskell Module Browser''; these have varying
support for user supplied documentation.

\subsubsection{Doing better}

As we saw earlier in this section, the two serious packages both run
into problems with simply lexing and printing in a proportional font,
although \program{lhs2TeX} compensates to some extent with a single alignment
column and also gives you the option of typesetting the code in a
monospaced font. But suppose instead of stopping after lexing the
Haskell we also parse it; this gives us a parse tree which we can
pretty-print with table-like environments to ensure that everything is
correctly indented.

\section{Parsing Haskell}\label{sec:parsing}

In this section we will start off by considering the high level
structure of the parser as a whole. We then take a look at the evolution
of each module separately, working on the assumption the input is a
single code block, and conclude with the changes that were necessary to
parse a module split into several sections as literate programs
generally are.

This section does not portray a true to life description of the
development process. In practise there was a reasonable amount of
overlap in the development of the various phases, but the description
offered here gives the best compromise between clarity of description
and reality.

\subsection{Parser structure}

This project can be broken into two halves---parsing Haskell and
pretty-printing the abstract syntax tree in {\LaTeX}. Let us start with
the first of these, parsing Haskell. Our parser will follow the
structure of the Haskell~98 report as closely as possible so as to make
it easy to compare the code to the report when debugging or applying
fixes made to the report to the code.

As noted in Section~\ref{sec:theory_parsing}, the job of parsing is in
general easiest done when broken into two parts---first the input is
lexed, then it is syntactically analysed. The Haskell~98 report has been
written with this in mind, and as such a ``lexical syntax'' and a
``context free syntax'' can be found in appendix B of the report. As the
intention is to follow the structure used in the report as closely as
possible, and given the many computer scientist years of experience in
this field, we will take this approach.

However, there are three things we have not yet managed to account for.
First is the need to extract the executable code from the surrounding
text. Although this project is intended to be used for scripts using the
second literate style described in the report~\cite{haskell98report}, it
is desirable to also cope with the first style and
plain~Haskell\footnote{Also sometimes refered to as illiterate Haskell.}.
As this defines the characters the lexer operates on we will do it
before the lexing stage.

Second, as in many languages, Haskell allows us to set the associativity
and binding precedence (together refered to as the fixity) of infix
operators. Fixity information refers to operators rather than individual
characters so common sense tells us to handle it after the lexing stage.
As the expression ``$a \oplus b \otimes c$'' produces different parse
trees depending on the fixities of $\oplus$ and $\otimes$ we had better
handle it before the syntax analysis stage.

The third is less common. Haskell allows some of the structure of a
script to be conveyed either by explicitly entering braces and
semicolons or with the positions of braces and semicolons implied by the
layout of the script; i.e., the whitespace in a Haskell script can be
significant. Both styles can be mixed in the same script and the
so-called ``layout rule'' describes where the implicit braces and
semicolons go. This description is in terms of tokens such as
\hkeyword{where} so it makes sense to do it after the lexer, but the syntax
analyser will need to know about the structure of the script, so we must
apply the layout rule between the two.

Thus we want to handle both fixity and layout between the lexer and
parser, which leaves the question of which we do first. Consider how the
scoping of fixity information works in Haskell; a fixity definition
applies to the operator of the same name defined in the same declaration
group\footnote{Two declarations are in the same declaration group if
there is a block (a list of definitions surrounded by \{ and \})
directly containing both declarations (i.e. they are not in a nested
group).}. This operator definition, and thus also the fixity information
associated with it, is then used in the current block and all subblocks
until a more local declaration of an operator with the same name
replaces it. As the fixity information needs to know where the blocks
are it is only logical to apply the layout rule first.

Now we have a high level overview for what is conceptually\footnote{As
Haskell is a lazy language the evaluation order will not actually mean
the five stages get run sequentially, but for most purposes we can treat
it as if they were.} a five pass parser:

\begin{displaymath}
\textit{parser} = \textit{syntax\_analyse} \circ \textit{fixity} \circ
        \textit{layout} \circ \textit{lex} \circ \textit{extract\_code}
\end{displaymath}

\noindent We will implement each stage as a module, applying the
principles of abstraction and separation of concerns liberally. Where
appropriate stages will be further broken down into sub modules, and
some modules may be shared between different stages.

\subsection{Unlitting}

Ultimately the aim is for the parser to be able to cope with multiple
sections of code, either in sections delimited by \verb!\begin{code}!
and \verb!\end{code}! or blocks of lines beginning with a \verb!>!, and
substitute nicely typeset equivalents with the surrounding text
untouched.

However, for a first approximation we will simply unlit the script,
i.e., extract all the code as a single chunk and throw away the
surrounding text. The report also requires that a line containing only
white space be between any surrounding text and a program line beginning
with a `\verb!>!' character. For now we will ignore this restriction,
returning to it when we write the final implementation of this phase.

This simplified process can be fairly straightforwardly coded as shown
in Figure~\ref{fig:unlit_first_draft_code}.

\begin{figure}
\begin{code}
unlit :: String -> String
unlit = concat . unlit_lines . lineify

unlit_lines :: [String] -> [String]
unlit_lines [] = []
unlit_lines (('>':xs):xss) = (' ':xs):unlit_lines xss
unlit_lines (xs:xss)
 | take 12 xs == "\\begin{code}" = if ok then block ++ unlit_lines rest
                                         else fail
 where (block, rest) = get_block xss
       ok = all isSpace $ drop 12 xs
       fail = error "\\begin{code} not followed by whitespace"
unlit_lines (_:xss) = unlit_lines xss

get_block :: [String] -> ([String], [String])
get_block [] = error "\\end{code} missing"
get_block (xs:xss)
 | take 10 xs == "\\end{code}" = if ok then ([], xss)
                                       else fail
 where ok = all isSpace $ drop 10 xs
       fail = error "\\end{code} not followed by whitespace"
get_block (xs:xss) = (xs:block, rest)
  where (block, rest) = get_block xss

lineify :: String -> [String]
lineify "" = []
lineify xs = this:lineify rest
  where (this, rest) = get_line xs

get_line :: String -> (String, String)
get_line "" = ("", "")
get_line ('\n':s) = ("\n", s)
get_line ('\r':s) = case s of
                        '\n':s' -> ("\r\n", s')
                        _ -> ("\r", s)
get_line (c:s) = (c:ys, zs)
  where (ys, zs) = get_line s
\end{code}
\caption{\hvarid{unlit} first draft code}
\label{fig:unlit_first_draft_code}
\end{figure}

\subsection{Parser Combinators}\label{sec:PCs}

\subsubsection{Existing PC libraries}

In order to write the lexer we shall use Parser Combinators (PCs). There
are two existing PC libraries for Haskell available, UU PC lib and
parsec. However, both of these use predictive parsing in order to
avoid backtracking and guarantee $O(n)$ time complexity for a given
parser on an input of length $n$. While this is an admirable quality, it
means that the set of grammars they allow has to be more restrictive
than we would like.

\begin{figure}
\subfigure[Grammar in Haskell~98 report]{
\begin{tabular}{lcl}
\textit{integer\/}&$\rightarrow$&\textit{decimal\/}\cr
&$\mid$&\texttt{0o} \textit{octal\/} $\mid$ \texttt{0O} \textit{octal\/}\cr
&$\mid$&\texttt{0x} \textit{hexadecimal\/}
 $\mid$ \texttt{0X} \textit{hexadecimal\/}\cr
\textit{decimal\/}&$\rightarrow$&\textit{digit\/} \{\textit{digit\/}\}\cr
\textit{octal\/}&$\rightarrow$&\textit{octit\/} \{\textit{octit\/}\}\cr
\textit{hexadecimal\/}&$\rightarrow$&\textit{hexit\/} \{\textit{hexit\/}\}\cr
\textit{digit\/}&$\rightarrow$&\texttt{0} $\mid$ \texttt{1} $\mid$ \dots
                                          $\mid$ \texttt{9}\cr
\textit{octit\/}&$\rightarrow$&\texttt{0} $\mid$ \texttt{1} $\mid$ \dots
                                          $\mid$ \texttt{7}\cr
\textit{hexit\/}&$\rightarrow$&\textit{digit\/} $\mid$ \texttt{A} $\mid$ \dots
              $\mid$ \texttt{F} $\mid$ \texttt{a} $\mid$ \dots $\mid$ \texttt{f}\cr
\end{tabular}
\label{fig:integer_lexical_syntax}
}

\subfigure[Predictive grammar]{
\begin{tabular}{lcl}
\textit{integer\/}&$\rightarrow$&\textit{nzdigit\/} \textit{decimal'\/} $\mid$ \texttt{0} \textit{integer'\/}\cr
\textit{integer'\/}&$\rightarrow$&\textit{decimal'\/}\cr
&$\mid$&\texttt{o} \textit{octal\/} $\mid$ \texttt{O} \textit{octal\/}\cr
&$\mid$&\texttt{x} \textit{hexadecimal\/} $\mid$ \texttt{X} \textit{hexadecimal\/}\cr
\textit{decimal'\/}&$\rightarrow$&\{\textit{digit\/}\}\cr
\textit{octal\/}&$\rightarrow$&\textit{octit\/} \{\textit{octit\/}\}\cr
\textit{hexadecimal\/}&$\rightarrow$&\textit{hexit\/} \{\textit{hexit\/}\}\cr
\textit{digit\/}&$\rightarrow$&\texttt{0} $\mid$ \texttt{1} $\mid$ \dots
                                          $\mid$ \texttt{9}\cr
\textit{nzdigit\/}&$\rightarrow$&\texttt{1} $\mid$ \texttt{2} $\mid$ \dots
                                            $\mid$ \texttt{9}\cr
\textit{octit\/}&$\rightarrow$&\texttt{0} $\mid$ \texttt{1} $\mid$ \dots
                                          $\mid$ \texttt{7}\cr
\textit{hexit\/}&$\rightarrow$&\textit{digit\/} $\mid$ \texttt{A} $\mid$ \dots
                 $\mid$ \texttt{F} $\mid$ \texttt{a} $\mid$ \dots $\mid$ \texttt{f}\cr
\end{tabular}
\label{fig:predictive_integer_lexical_syntax}
}
\caption{(Simplified) lexical syntax for integers}
\end{figure}

For example, consider the slightly simplified extract from the Haskell
lexical syntax in Figure~\ref{fig:integer_lexical_syntax}. Upon finding
a `\verb!0!' in the input a predictive parser built directly from this
grammar would immediately commit to the first production which allowed a
`\verb!0!', here \textit{decimal\/}. This means that it will never
recognise an octal or hexadecimal constant! We can translate this
grammar into an equivalent one suitable for use with a predictive
parser; such a grammar is shown in
Figure~\ref{fig:predictive_integer_lexical_syntax} and algorithms exist to
perform such a translation for any grammar.

However, the resulting grammar would bear relatively little resemblance
to the grammar in the report which would make it difficult to check its
correctness by hand and to update it with new features or fixes. Another
example is shown in Figures~\ref{fig:lexical_syntax_reservedid_in}
and~\ref{fig:predictive_lexical_syntax_reservedid_in}. We must also
remember that these are simplified extracts of just a small part of the
lexical syntax---the effect would be far worse if we were considering it
in its entirety. Furthermore we have not gained anything in terms of
time complexity, as we shall now demonstrate.

\begin{figure}
\subfigure[Grammar in Haskell~98 report]{
\begin{tabular}{lcl}
\textit{reservedid\/}&$\rightarrow$&\texttt{if} $\mid$ \texttt{in} $\mid$
\texttt{infix} $\mid$ \texttt{infixl} $\mid$ \texttt{infixr} $\mid$ \texttt{instance}\cr
\end{tabular}
\label{fig:lexical_syntax_reservedid_in}
}

\subfigure[Predictive grammar]{
\begin{tabular}{lcl}
\textit{reservedid\/}&$\rightarrow$&\texttt{i} \textit{reservedid\_i\/}\cr
\textit{reservedid\_i\/}&$\rightarrow$&f $\mid$ n \textit{reservedid\_in\/}\cr
\textit{reservedid\_in\/}&$\rightarrow$&\texttt{fix} \textit{reservedid\_infix\/} $\mid$ \texttt{stance} $\mid$ $\epsilon$\cr
\textit{reservedid\_infix\/}&$\rightarrow$&l $\mid$ r $\mid$ $\epsilon$\cr
\end{tabular}
\label{fig:predictive_lexical_syntax_reservedid_in}
}
\caption{Subset of lexical syntax for reserved IDs}
\end{figure}

Assume that checking that the next input character matches a given
symbol can be done in constant time and that expanding a non-terminal
into the union of its productions can also be done in constant time.
This means that we can also compare the next input character against an
arbitrary, but fixed, number of symbols in constant time. This is
sufficient to show that \textit{octit}, \textit{digit\/} and
\textit{hexit\/} take constant time for input of any length.

Now consider \textit{decimal}; each time we try to match an input
character with \textit{digit\/} it takes us constant time as we have
shown. If it succeeds then we repeat with one fewer input characters,
while if it fails then we stop. Therefore, given an input of length $n$,
we can do a constant amount of work at most $n$ times, so the amount of
work we do is $O\left(n\right)$. A similar argument shows that
\textit{octal} and \textit{hexadecimal} are $O\left(n\right)$.

Given a parser for \verb!0o! \textit{octal} and input of length $n$ we
would need constant time to match the first character against `\verb!0!',
constant time to match the second character against `\verb!o!' and the
remaining $n - 2$ characters will take $O\left(n - 2\right)$ time.
Therefore the whole parser is also $O\left(n\right)$. Similar arguments
hold for the other alternatives of the \textit{integer} non-terminal.

Finally we consider \textit{integer}. Each of the alternatives will take
$O\left(n\right)$ time so we can do all five in $O\left(n\right)$ time,
proving our assertion that we can lex integers in linear time even
though our parser combinators can backtrack.

In order to show that parsing a given grammar using backtracking is
linear it is sufficient to show that none of the right hand sides of the
productions for a non-terminal $\alpha$ can, after expanding an
arbitrary number of further productions, contain an $\alpha$. In other
words it is sufficient for the graph of the productions to be acyclic.
We can see that this is the case for the lexical syntax in the report,
so we will be able to lex it in linear time using a backtracking PC
library. This is backed up by experimental data from the finished lexer
shown in Figure~\ref{fig:exp_lex_time}. An alternative method for fast
lazy lexing is explored in~\cite{LLiF}.

\begin{figure}
This table shows the time taken for a machine with a 733MHz Intel
Celeron CPU and 512MB of RAM to lex 8, 16 and 32 copies of a Haskell
script concatenated together respectively.

\begin{center}
\begin{tabular}{|l|l|}
\hline
\textbf{Size}&\textbf{CPU time taken}\cr
\hline
355k&0m44.240s\cr
710k&1m29.760s\cr
1420k&3m07.750s\cr
\hline
\end{tabular}
\end{center}
\caption{Experimental data: Lexical analysis times}
\label{fig:exp_lex_time}
\end{figure}

Parsec does have a \hvarid{try} primitive which relaxes the restrictions
imposed by predictive parsing to an extent. However, it also means we
lose the guaranteed linear time and the code is no longer as clean. All
things considered the best approach seems to be to write our own parser
combinator library specially designed for our task.

\subsubsection{Preparing for our own PC library}

When writing our parser combinator library we will borrow many of the
names from the UU PC library. In some cases this will mean the same
function or operator has slightly different semantics in the two
libraries, in particular the Choice operator. We hope that this will not
cause too much confusion.

Before we start to think about implementing our primitives there is one
more key point we must pick up from the report.

\begin{quote}
In both the lexical and the context-free syntax, there are some
ambiguities that are to be resolved by making grammatical phrases as
long as possible, proceeding from left to right (in shift-reduce
parsing, resolving shift/reduce conflicts by shifting). In the lexical
syntax, this is the "maximal munch" rule. In the conText-free syntax,
this means that conditionals, let-expressions, and lambda abstractions
extend to the right as far as possible.
\end{quote}

This means that we also need to know how many input symbols our parsers
consume so that, when we have the choice of two parsers which both
succeed, we can pick the one which consumes the most input.

\subsubsection{Our own PC library: First draft}
% BEGIN first_draft/PCbase.lhs
%module PCbase (Parser, pFail, pSucceed, pPred, (<*>), (<|>), notAcceptsEmpty) where

The first step in the actual implementation of the first draft of our PC
library is to consider what the type of a parser should be. We
use a type synonym partly to simplify error messages given by our
compiler, but mainly because it means we have less to change as the
system evolves. In particular, the derived parsers will not need to be
changed at all as we add functionality to the primitives.

The minimum information a parser needs to provide us with is whether it
succeeded and, if it did, what it returned and how much of the input it
consumed. We use the \htycon{Maybe} monad to indicate success or failure and, for
convenience, also return the rest of the input. Thus
\begin{code}
type Parser a = String -> Maybe (a, Int, String)
\end{code}
\noindent is a (very simple) first approximation. However, we will want
to use our library for both lexing and syntax analysinging and, for the
latter, the input will generally not be a list of characters but a list
of tokens. Thus we generalise this slightly to a type synonym taking two
type variables, one for the input type and one for the output type:
\begin{code}
type Parser a b = [a] -> Maybe (b, Int, [a])
\end{code}

This done we start to define our primitives, referring to
Section~\ref{sec:parser_combinators}, the existing
implementations~\cite{uuparsingcombinators,parsec} and the
tutorial~\cite{MPCs} for an idea of what we want to be able to do.

We start with the Fail and Succeed primitives which are trivial to
define, as shown in Figure~\ref{fig:pFail_pSucceed_first_draft}. We
pause only briefly to note that \hvarid{pSucceed} consumes 0 characters.

\begin{figure}
\begin{code}
pFail :: Parser a b
pFail = \inp -> Nothing
pSucceed :: b -> Parser a b
pSucceed v = \inp -> Just (v, 0, inp)
\end{code}
\caption{\hvarid{pFail} and \hvarid{pSucceed} first draft}
\label{fig:pFail_pSucceed_first_draft}
\end{figure}

We note that Match is not primitive in the UU~parsing
combinators\cite{uuparsingcombinators}, but is defined in terms of the
${<}..{>}$ operator, which succeeds if the next symbol is between its
two operands inclusive, as defined by the \htycon{Ord} operator $\le$. The
only places where this looks like it might be useful are where we match
the upper and lower case letters, digits, octits and hexits. Even then
the prelude supplies us with predicates \hvarid{isUpper}, \hvarid{isLower},
\hvarid{isDigit}, \hvarid{isOctDigit} and \hvarid{isHexDigit}. Thus a more
logical choice of primitive for us is \hvarid{pPred} which, given a
predicate, returns a parser that succeeds if and only if there is at
least one more symbol and the predicate holds for that symbol. If
successful it consumes that symbol, removing it from the input and
returning it. The fairly straightforward code is shown in
Figure~\ref{fig:pPred_first_draft}.

\begin{figure}
\begin{code}
pPred :: (a -> Bool) -> Parser a a
pPred f = \inp -> case inp of
                      [] -> Nothing
                      (t:ts) -> if f t
                                then Just (t, 1, ts)
                                else Nothing
\end{code}
\caption{\hvarid{pPred} first draft}
\label{fig:pPred_first_draft}
\end{figure}

We will use the application style of Sequential Composition described in
Section~\ref{sec:parser_combinators}. A flowchart describing the
operator is given in Figure~\ref{fig:seq_comp_flowchart} and can be
simply translated into the code in Figure~\ref{fig:seq_comp_code}.

\begin{figure}
\includegraphics{seq}
\caption{Flowchart describing sequential composition of parsers}
\label{fig:seq_comp_flowchart}
\end{figure}

%\begin{itemize}
%\item{The first parser (p1) is applied to the input (inp)}
%\item{If the first parser failed then fail.}
%\item{Let the result be v1, the number of characters consumed
%be n1 and the remaining input be inp'}
%\item{The second parser (p2) is applied to the remaining input (inp')}
%\item{If the second parser failed then fail.}
%\item{Let the result be v2, the number of characters consumed be n2 and the remaining input be $inp'{}'$}
%\item{Successfully return $v1$ applied to $v2$, reporting $n1 + n2$ characters consumed and $inp'{}'$ input remaining}
%\end{itemize}

\begin{figure}
\begin{code}
(<*>) :: Parser a (b -> c) -> Parser a b -> Parser a c
p1 <*> p2 = \inp -> case p1 inp of
                        Nothing -> Nothing
                        Just (v1, n1, inp1) ->
                            case p2 inp1 of
                                Nothing -> Nothing
                                Just (v2, n2, inp2) ->
                                    Just (v1 v2, n1 + n2, inp2)
\end{code}
\caption{Code for sequential composition of parsers}
\label{fig:seq_comp_code}
\end{figure}

Our definition of choice operator is influenced strongly by the Haskell
report which, as quoted earlier, says ``ambiguities that are to be
resolved by making grammatical phrases as long as
possible''~\cite[appendix B, section 1]{haskell98report}. In other
words, if both parsers succeed we take the one that consumed the most
input. If both fail then we must also fail and, as you would expect, if
only one succeeds then we successfully return the value it returned. Of
course, it is possible for both parsers to succeed \emph{and consume the
same amount of input}. However, the report does not allow this to
happen, so if it does then we fail. The code is given in
Figure~\ref{fig:choice_code}.

\begin{figure}
\begin{code}
(<|>) :: Parser a b -> Parser a b -> Parser a b
p1 <|> p2 = \inp -> case (p1 inp, p2 inp) of
                        (Nothing, r2) -> r2
                        (r1, Nothing) -> r1
                        (r1@(Just (_, n1, _)), r2@(Just (_, n2, _)))
                            -> case n1 `compare` n2 of
                                   GT -> r1
                                   LT -> r2
                                   EQ -> error same
    where same = "Both operands of <|> consumed same number of symbols"
\end{code}
\caption{Code for choice of parsers}
\label{fig:choice_code}
\end{figure}
% END first_draft/PCbase.lhs

By default the operators ${<}{*}{>}$ and ${<}{|}{>}$ will be considered
left associative and have the highest fixity, i.e., they will bind as
tightly as possible. As ${<}{*}{>}$ is roughly analogous to function
application it makes sense for it to be left associative for the same
reasons that juxtaposition is. The Choice operator is commutative so it
makes no difference whether ${<}{|}{>}$ is left or right associative; we
arbitrarily decide to make it left associative. It makes sense for one
of the operators to bind tighter than the other to reduce the number of
brackets required. By making ${<}{*}{>}$ bind tighter we would allow
ourselves to easily write a choice of multiple parsers and would also be
following the same precedence rules as BNF, so we choose to do this. To
allow space for derived operators inbetween we define the binding
precedence of ${<}{*}{>}$ to be 7 and of ${<}{|}{>}$ to be 3.

This completes the first draft of our PC primitives. Before we test it
we will find it convenient to augment our library with a pair of derived
functions. The first is the \hvarid{pSym} function, defined in terms of
\hvarid{pPred} as discussed earlier. Second we observe we will often want
to apply a fixed data constructor or function to the result of one or
more parsers; we define ${<}{\$}{>}$ so that we do not need to clutter up
our code with many \hvarid{pSucceed} applications. Ths code for both is
shown in Figure~\ref{fig:code_psym}.

% BEGIN first_draft/PC.lhs
%module PC (module PCbase, pSym, (<$>)) where
%import PCbase

\begin{figure}
\begin{code}
pSym :: Eq a => a -> Parser a a
pSym = pPred . (==)
(<$>) :: (b -> c) -> Parser a b -> Parser a c
f <$> p = pSucceed f <*> p
\end{code}
\caption{Code for \hvarid{pSym} and ${<}{\$}{>}$}
\label{fig:code_psym}
\end{figure}

% END first_draft/PC.lhs

As the Haskell lexical syntax includes constructs like lists of digits
we would be foolish to attempt to construct a lexer before implementing
some derived library functions to deal with such constructs; however, it
is still wise to test what we have done thus far so that we can check
that our library is behaving as expected. The test session is shown in
Figure~\ref{fig:first_draft_basic_pc_library}. As we can see the parser
is correctly accepting an `\verb!a!' followed by either a `\verb!b!' or
a `\verb!c!' and returning the pair of what it consumes if it is
successful.

\begin{figure}
\begin{example}
Hugs session for:
/usr/share/hugs98/lib/Prelude.hs
PCbase.lhs
PC.lhs
Type :? for help
PC> \hbox{\textbf{((,) <$> pSym 'a' <*> (pSym 'b' <|> pSym 'c')) "abcde"}}
Just (('a','b'),2,"cde")
PC> \hbox{\textbf{((,) <$> pSym 'a' <*> (pSym 'b' <|> pSym 'c')) "ac"}}
Just (('a','c'),2,"")
PC> \hbox{\textbf{((,) <$> pSym 'a' <*> (pSym 'b' <|> pSym 'c')) ""}}
Nothing
PC> \hbox{\textbf{((,) <$> pSym 'a' <*> (pSym 'b' <|> pSym 'c')) "q"}}
Nothing
PC> \hbox{\textbf{((,) <$> pSym 'a' <*> (pSym 'b' <|> pSym 'c')) "a"}}
Nothing
\end{example}
\caption{Testing first draft basic PC library}
\label{fig:first_draft_basic_pc_library}
\end{figure}

Note that Figure~\ref{fig:first_draft_basic_pc_library} shows that two
scripts have been loaded, \filename{PCbase.lhs} and \filename{PC.lhs}. All the
functions which need to know how a \htycon{Parser} $\alpha$ is
constructed, i.e., the primitives, are in \filename{PCbase.lhs}. The derived
functions are all in \filename{PC.lhs}, which imports and re-exports all of
\filename{PCbase.lhs}. This abstraction means that the set of functions we
need to check when altering how the parsers work is exactly the set of
functions in a single file and, while we currently trust the derived
functions not to peek at the internals of the \htycon{Parser} type, when we later
hide the implementation details with a datatype whose constructors are
not exported, the derived functions, and parsers built with the library,
will be \emph{forced} to obey the published interface.

\subsubsection{Our own PC library: Augmenting the first draft}

If we are to use the PCs then we will need to know whether our parser
was successful, what the return value was and whether or not it consumed
the entire input. For the last we instead provide a function which
returns the remaining input to allow more flexibility. All three require
knowledge of the internals of the \htycon{Parser} type so must be placed
in \filename{PCbase.lhs}. The code for each is straightforward; see
Figure~\ref{fig:parsed_info_first_draft}.

\begin{figure}
\begin{code}
parse_succeeded :: Maybe (b, Int, [a]) -> Bool
parse_succeeded (Just _) = True
parse_succeeded Nothing = False
ret_val :: Maybe (b, Int, [a]) -> b
ret_val (Just (ret, _, _)) = ret
ret_val Nothing = error "Tried to take ret_val of a failed parse"
rest_of_input :: Maybe (b, Int, [a]) -> [a]
rest_of_input (Just (_, _, rest)) = rest
rest_of_input Nothing = error "Tried to get rest_of_input of a failed parse"
\end{code}
\caption{Functions for information about a parse}
\label{fig:parsed_info_first_draft}
\end{figure}

As mentioned earlier there are many more constructs, such as lists, that
will be useful to us as we build our lexer and syntax analyser. Before
we go any further let us augment our library by adding more derived
parsers and, when necessary, more primitives.

Two more derived parsers, ${<}{\$}$ and ${<}{*}$, from the UU PC lib are
equivalent to ${<}{\$}{>}$ and ${<}{*}{>}$ respectively except that they
ignore the result of the parser in their second argument. They can
trivially be defined in terms of existing functions. Both use
\hvarid{const} with the existing function to drop the value of the second
operand; the code is shown in Figure~\ref{fig:drop_second} and an
example showing how they can be used to drop parentheses matched is in
Figure~\ref{fig:drop_second_example}.

\begin{figure}
\begin{code}
(<$) :: b -> Parser a c -> Parser a b
p <$  q = const p <$> q
(<*) :: Parser a b -> Parser a c -> Parser a b
p <* q = const <$> p <*> q
\end{code}
\caption{Code for ${<}{\$}$ and ${<}{*}$}
\label{fig:drop_second}
\end{figure}

\begin{figure}
\begin{code}
p_paren_string :: Parser Char String
p_paren_string = id
             <$  pSym '('
             <*> pList 0 (pPred (')' /=))
             <*  pSym ')'
\end{code}
\caption{Example demonstrating the use of ${<}{\$}$ and ${<}{*}$}
\label{fig:drop_second_example}
\end{figure}

In addition to \hvarid{pSym} we define \hvarid{pSyms}, which matches
multiple symbols; it can also be easily defined as a derived function.
It takes a list of symbols and produces a parser which accepts the
complete list of symbols in order. To achieve this we fold a function
which uses \hvarid{pSym} to parse each individual symbol and constructs a
list from the results. A requirement that the type $\alpha$ be an
instance of the \htycon{Eq} class is inherited from \hvarid{pSym}. The code
is shown in Figure~\ref{fig:pSyms_first_draft}.

\begin{figure}
\begin{code}
pSyms :: (Eq a) => [a] -> Parser a [a]
pSyms = foldr (\s p -> (:) <$> pSym s <*> p) (pSucceed [])
\end{code}
\caption{Code for \hvarid{pSyms}}
\label{fig:pSyms_first_draft}
\end{figure}

We will also want to parse lists of construct. To produce a list we
repeatedly apply a parser to the input until it fails, returning all the
successful results in a list. Note that we must ensure that the parser
cannot succeed without consuming any input or it would succeed
forever!\footnote{In actual fact the way we implement it means we would
get an error from ${<}{|}{>}$ as both of the operand parsers will
successfully consume the same number, i.e., 0, symbols.} To prevent this
we first try applying the parser to the empty input list and, using
\hvarid{parse\_succeeded} to check, give an error if it succeeds.

As well as a parser, our list function will take an \htycon{Int} as an argument.
This specifies a minimum number of successful parses we require, i.e.,
the minimum length of the resulting list. While this is non-zero we
apply the parser and cons the result to a recursive call requiring one
fewer elements. The list with at least 0 elements is the list with at
least 1 element or the empty list. This algorithm can be
straightforwardly converted to code, producing the derived function
shown, along with the (also derived) helper functions
\hvarid{accepts\_empty} and \hvarid{opt}, in
Figure~\ref{fig:pList_first_draft}. We use \hvarid{opt} as an infix
operator; we have given it a binding precedence of 4, so the only
operator it binds tighter than is ${<}{|}{>}$, and made it
non-associative as neither left nor right associativity make sense. The
guard against the parser successfully consuming 0 characters in
\hvarid{opt} is not necessary for \hvarid{pList} but we will see that it is
useful elsewhere shortly.

\begin{figure}
\begin{code}
accepts_empty :: Parser a b -> Bool
accepts_empty p = parse_succeeded (p [])

opt :: Parser a b -> b -> Parser a b
p `opt` v = p <|> if accepts_empty p then pFail else pSucceed v

pList :: Int -> Parser a b -> Parser a [b]
pList _ p | accepts_empty p = error "Tried to make a list of empties"
pList 0 p = pList 1 p `opt` []
pList (i+1) p = (:) <$> p <*> pList i p
\end{code}
\caption{Code for \hvarid{pList} and helper functions \hvarid{accepts\_empty} and \hvarid{opt}}
\label{fig:pList_first_draft}
\end{figure}

These simple lists are just the tip of the iceberg, however. There are
several constructs in the Haskell context free grammar which are a list
of some construct with another ``separator'' construct interspersed; a
final separator is sometimes allowed at the end of the list. As well as
a parser for the ``separators'' and a parser for the ``things'' we will
also pass a minimum number of ``things'' we require, as in \hvarid{pList},
and a boolean value which is \htycon{True} iff a trailing separator is
allowed.

If $i+1$ ``things'' are required and $i$ is at least $0$ then we require
a ``thing'' followed by a normal list of at least $i$ separators
followed by ``things''. Finally we can have an optional trailing
separator if we are allowed. It is here that the extra guard in
\hvarid{opt} mentioned earlier becomes necessary in case the separator
parser can successfully consume no input.

If 0 ``things'' are required then we either act as if 1 ``thing'' was
required, match a single separator if a trailing separator is allowed or
simply return the empty list. We only allow the last case if the parser
for ``things'' cannot successfully consume 0 characters; otherwise it is
possible that the first and last cases would both successfully consume 0
characters.

The final thing to note is that, before anything else is considered, we
first check that the separator parser and the ``thing'' parser cannot
both successfully consume 0 symbols. While we could let \hvarid{pList}
check this for us we can give a more useful error message by doing so
ourselves. The complete code is shown in
Figure~\ref{fig:pSList_first_draft}. When the ``separator'' is a
semicolon we want the value returned by the parser as it will tell us if
the semicolon was inserted by the layout rule or explicitly by the
programmar; however, if the ``separator'' is just a comma, which is
always explicitly inserted, then we do not care about the value. We
define a functions \hvarid{get\_rights}, shown in the same figure, which we
use when we do not care about the values of the separators.

\begin{figure}
\begin{code}
pSList :: Bool -> Int -> Parser a b -> Parser a c -> Parser a ([Either b c])
pSList _ _ p_s p_t
 | accepts_empty p_s && accepts_empty p_t
  = error "p_s and p_t can't both accept empty in pSList"
pSList end 0 p_s p_t = pSList end 1 p_s p_t
                   <|> (if end then wrap . Left <$> p_s else pFail)
                   <|> if accepts_empty p_t
                       then pFail
                       else pSucceed []
pSList end (i+1) p_s p_t
 =     (\h mid t -> h ++ concat mid ++ t)
   <$> (wrap . Right <$> p_t)
   <*> (pList i ((\s t -> [Left s, Right t]) <$> p_s <*> p_t))
   <*> if end then wrap . Left <$> p_s `opt` []
              else pSucceed []
get_rights :: [Either a b] -> [b]
get_rights [] = []
get_rights (Left _:es) = get_rights es
get_rights (Right r:es) = r:get_rights es
\end{code}
\caption{Code for \hvarid{pSList} and \hvarid{get\_rights}}
\label{fig:pSList_first_draft}
\end{figure}

We will need to be able to do things such as excluding keywords from the
parser for function names. To this end we define a new primitive
operator ${<}{!}{>}$ such that \hbox{\hvarid{p} ${<}{!}{>}$ \hvarid{q}} succeeds with the value
returned by \hvarid{p} if \hvarid{q} either fails or succeeds but consumes less input;
otherwise it fails. The code is given in
Figure~\ref{fig:first_draft_bang}. As we will generally want to exclude
a set of parses from an entire parser we give ${<}{!}{>}$ a precedence of
2, the lowest of all our operators, and decide to make it left
associative as successive restrictions seem more likely than a
restricted restriction.

\begin{figure}
\begin{code}
(<!>) :: Parser a b -> Parser a c -> Parser a b
p <!> q = \inp -> case (p inp, q inp) of
                      (p_res, Nothing) -> p_res
                      (Nothing, _) -> Nothing
                      (Just sp@(_, np, _), Just (_, nq, _))
                          | nq >= np -> Nothing
                          | otherwise -> Just sp
\end{code}
%
%\begin{code}
%(<!>) :: Parser a b -> Parser a c -> Parser a b
%p <!> q = \inp -> case q inp of
%                      Nothing -> p inp
%                      Just (_, nq, _) ->
%                          case p inp of
%                              Nothing -> Nothing
%                              Just sp@(_, np, _) ->
%                                  if nq >= np
%                                  then Nothing
%                                  else Just sp
%\end{code}
\caption{Code for ${<}{!}{>}$}
\label{fig:first_draft_bang}
\end{figure}

Two more small functions remain before we have finally completed our
first draft, both to do with optional constructs. The first takes a
predicate and consumes the first input character if the predicate holds
for it, returning \htycon{True}; otherwise it just returns \htycon{False}.
The second applies a parser and, if it succeeds, returns the result
wrapped up with the \htycon{Just} data constructor; otherwise it just
returns \htycon{Nothing}. It first checks that the parser passed cannot
succeed without consuming any input in order to try to protect us from
ourselves; if this is not the case then \htycon{pMaybe} can be replaced by
\htycon{Just} ${<}\${>}$. The code for both is given in
Figure~\ref{fig:pPredHolds_pMaybe_first_draft}.

\begin{figure}
\begin{code}
pPredHolds :: (a -> Bool) -> Parser a Bool
pPredHolds f = True <$  pPred f `opt` False

pMaybe :: Parser a b -> Parser a (Maybe b)
pMaybe p = if accepts_empty p
           then error "pMaybe passed parser that accepts empty"
           else Just <$> p `opt` Nothing
\end{code}
\caption{Code for \hvarid{pPredHolds} and \hvarid{pMaybe}}
\label{fig:pPredHolds_pMaybe_first_draft}
\end{figure}

\subsubsection{Error handling: knowing where we are}

Currently, if we come across an error we just fail. There are two good
reasons for providing more information on where the error occurred; the
first is that people using a parser built upon our library will want to
know where the errors in their input are so that they can fix
them. Secondly, and perhaps more important from our point of view, we
will find error reporting useful as a means of finding errors in the
parsers we build when we find they fail with valid input! We will
provide the user with the line and character where we find the first
error which should provide sufficient information for them to be able to
pinpoint and correct the problem. Used repeatedly all bugs, whether in
the parser combinator library, the parser built on it or the input being
parsed, can be fixed.

A position in a script is either a line and character number or the
special position ``end of file''. A sufficient definition of
\htycon{Position} would be:
\begin{code}
type Position = Maybe (Int, Int)
\end{code}
\noindent with \htycon{Nothing} being used to represent the end of the
file but, in the interests of data abstraction and clarity of data
constructors, we will create our own datatype and type synonyms:
\begin{code}
type Line = Int
type Character = Int
data Position = Position !Line !Character
              | End_Of_File
    deriving Eq
\end{code}

\noindent The `\verb"!"'s specify that the \htycon{Position} constructor
is strict in its arguments; this does not affect the functionality but,
as profiling shows, improves the performance.

If we have two positions of possible errors, perhaps from both operands
of a Choice failing, then the chances are the error is at the position
furthest into the file---for example, if we give a parser that matches
either ``\verb!hello!'' or ``\verb!world!'' the input ``\verb!helli!''
then it could return an error at either character 5 or character 1, but
the error is clearly at character 5. The solution is to define an order
on \htycon{Position}s, which we do by making \htycon{Position} an
instance of Haskell's \htycon{Ord} class; the reason we made
\htycon{Position} derive \htycon{Eq} earlier as it is a prerequisite for
the \htycon{Ord} class. We then export a function \hvarid{furthest\_pos}
that takes two \htycon{Position}s and returns the further of the two.
The code is shown in Figure~\ref{fig:furthest_pos_code}.

\begin{figure}
\begin{code}
instance Ord Position where
    (Position l1 c1) <= (Position l2 c2) = (l1, c1) <= (l2, c2)
    _ <= End_Of_File = True
    End_Of_File <= _ = False

furthest_pos :: Position -> Position -> Position
furthest_pos = max
\end{code}
\caption{Code for \hvarid{furthest\_pos} and definition of \htycon{Ord} \htycon{Position}}
\label{fig:furthest_pos_code}
\end{figure}

Each input character will be paired with its position. The easiest way
to both correctly give each character its position and to unlit the
input is to do both at the same time; to avoid too much repetition we
will not go into the gory details, but Section~\ref{sec:code_blocks}
gives an overview of the final implementation. We will also need to
refer to the end of the file position at times, for example when
reporting an error after running out of input, and thus positions. With
our data structure \hvarid{end\_of\_file} is very simple, as shown in
Figure~\ref{fig:end_of_file}.

\begin{figure}
\begin{code}
end_of_file :: Position
end_of_file = End_Of_File
\end{code}
\caption{Code for \hvarid{end\_of\_file}}
\label{fig:end_of_file}
\end{figure}

\subsubsection{Error handling and the PC primitives}

A na\"{\i}ve implementation would just have the primitives report
failure at the point at which they fail, i.e., the type \htycon{Parser}
would be as given in Figure~\ref{fig:naive_error_parser_type} along with
the helper function \hvarid{num}, which extracts the number of characters
consumed by a successful parse. The primitives would be altered in the
obvious way; Figure~\ref{fig:naive_error_handling_PCs} gives some
examples along with the new helper function, \hvarid{get\_pos}, which gives
the position of the start of the input.

\begin{figure}
\begin{code}
type Parser a b = [(a, Position)] -> Parsed a b
data Parsed a b = Success b Int [(a, Position)]
                | Failure Position

num :: Parsed a b -> Int
num (Success _ n _) = n
num (Failure _) = error "num: Can't give num of a failure"
\end{code}
\caption{Na\"\i{}ve error handling \htycon{Parser} type and helper function \hvarid{num}}
\label{fig:naive_error_parser_type}
\end{figure}

\begin{figure}
\begin{code}
get_pos :: [(a, Position)] -> Position
get_pos [] = end_of_file
get_pos ((_, p):_) = p

pFail :: Parser a b
pFail = \inp -> Failure (get_pos inp)
pSucceed :: b -> Parser a b
pSucceed v = \inp -> Success v 0 inp

pPred :: (a -> Bool) -> Parser a a
pPred f = \inp -> case inp of
                      [] -> Failure end_of_file
                      ((t, p):tps) -> if f t
                                      then Success t 1 tps
                                      else Failure p

(<|>) :: Parser a b -> Parser a b -> Parser a b
p1 <|> p2 = \inp -> case (p1 inp, p2 inp) of
                        (Failure pos1, Failure pos2) ->
                            Failure (furthest_pos pos1 pos2)
                        (Failure _, r2) -> r2
                        (r1, Failure _) -> r1
                        (r1, r2)
                            -> case num r1 `compare` num r2 of
                                   GT -> r1
                                   LT -> r2
                                   EQ -> error same
    where same = "Both operands of <|> consumed same number of symbols"

(<*>) :: Parser a (b -> c) -> Parser a b -> Parser a c
p1 <*> p2 = \inp -> case p1 inp of
                        Failure pos1 -> Failure pos1
                        Success v1 n1 inp1 ->
                            case p2 inp1 of
                                Failure pos2 -> Failure pos2
                                Success v2 n2 inp2 ->
                                    Success v n inp2
                                        where v = v1 v2
                                              n = n1 + n2
\end{code}
\caption{Na\"\i{}ve error handling parser combinator primitives}
\label{fig:naive_error_handling_PCs}
\end{figure}

Now consider the example in Figure~\ref{fig:naive_error_handling_demo},
where \hvarid{posify} takes a list of characters and pairs each one with
its position, starting from character 1 on line 1, and we have derived
\htycon{Show} on all the data types so we can see the results. The
parser used accepts the string ``\verb!abcd!'' repeated any number of
times and surrounded by braces. To see why this example is relevant we
could imagine `\verb!a!' to be loosely representative of a function
left hand side, `\verb!b!' of the equals sign, `\verb!c!' of the
right hand side and `\verb!d!' of a \hkeyword{where} clause; the
parser is then loosely representative of a syntax analyser for a block
of function definitions.

\begin{figure}
\begin{example}
Hugs session for:
/usr/share/hugs98/lib/Prelude.hs
Position.lhs
PCbase.lhs
PC.lhs
PC> \hbox{\textbf{id <$ pSym '\{' <*> pList 0 (pSyms "abcd") <* pSym '\}' $ posify "\{abcdabcdabcd\}"}}
Success ["abcd","abcd","abcd"] 14 []
PC> \hbox{\textbf{id <$ pSym '\{' <*> pList 0 (pSyms "abcd") <* pSym '\}' $ posify "\{abcdabcdabd\}"}}
Failure (Position 1 10)
\end{example}
\caption{Na\"\i{}ve error handling demonstration}
\label{fig:naive_error_handling_demo}
\end{figure}

The first parse succeeds as expected, consuming all the input. However,
in the second parse we have left the third `\verb!c!' out of the input.
Our parser tells us there was an error on the first (and, in this case,
only) line at the tenth character, but we want to be told the error is
at the 12th character, i.e., the `\verb!d!' where we were expecting a
`\verb!c!'.

To see the problem consider what happens as the string gets parsed.
First the `\verb!{!' is matched and then a list of ``\verb!abcd!''s is
required. The first two ``\verb!abcd!''s are successfully matched but the
remaining input, ``\verb!abd}!'', does not match, so the \hvarid{pList}
function instead returns the empty list. The final parser is then
applied to the remaining input and fails to match a `\verb!}!' so the
position of the `\verb!a!' is returned as the error point. The key point
to note though is that the \hvarid{pList} function successfully consumed
two characters in one of its alternatives before that alternative failed
and the other alternative succeeded having consumed no input. Thus what
we must do is to remember the furthest point we reach and fail at
\emph{even if we succeed}. We are now ready to implement the second
draft of our PC library.

\subsubsection{Our own PC library: Second draft}

We now augment our definition of \htycon{Parsed} from the previous section
with a \htycon{Position} in the \htycon{Success} constructor as discussed.
This definition, which we will use for the second draft, is shown in
Figure~\ref{fig:parser_type_helper_functions_second_draft} along with
some helper functions we will use to rewrite the primitives. Note that
the existing derived parsers will continue to work once we have updated
the primitive parsers---this is a good example of a case when first
class modules would be useful---we could pass one of the modules of
primitive parsers as a parameter to the derived module.

\begin{figure}
\begin{code}
data Parsed a b = Success (b, Int, [(a, Position)], Position)
                | Failure Position
type Parser a b = [(a, Position)] -> Parsed a b
num :: Parsed a b -> Int
num (Success (_, n, _, _)) = n
num (Failure _) = error "num: Can't give num of a failure"
rinp :: Parsed a b -> [(a, Position)]
rinp (Success (_, _, r, _)) = r
rinp (Failure _) = error "rinp: Can't give remaining input of a failure"
pos :: Parsed a b -> Position
pos (Success (_, _, _, p)) = p
pos (Failure p) = p
\end{code}
\caption{Second draft \htycon{Parser} type and helper functions}
\label{fig:parser_type_helper_functions_second_draft}
\end{figure}

The updated primitive parsers are given in
Figure~\ref{fig:primitive_parsers_second_draft}. The first three are
straightforward, but the Sequential Composition, Choice and Exclusion
operators need slightly more modification to choose the correct position
in all cases. Also note that we have used `\hvarid{seq}` and ${\$}{!}$ to
force evaluation of the furthest position reached; without this the tree
of unevaluated thunks consumes a large amount of heap, as discovered by
the profiling features of GHC.

\begin{figure}
\begin{code}
pFail :: Parser a b
pFail = \inp -> Failure (get_pos inp)
pSucceed :: b -> Parser a b
pSucceed v = \inp -> Success (v, 0, inp, get_pos inp)

pPred :: (a -> Bool) -> Parser a a
pPred f = \inp -> case inp of
                      [] -> Failure end_of_file
                      ((t, p):inp') -> if f t
                                       then Success (t, 1, inp', get_pos inp')
                                       else Failure p

(<|>) :: (Show a) => Parser a b -> Parser a b -> Parser a b
p <|> q
 = \inp -> case (p inp, q inp) of
               (Success s1, Success s2) -> case num s1 `compare` num s2 of
                                               GT -> succeed s1 (pos s2)
                                               LT -> succeed s2 (pos s1)
                                               EQ -> Failure (get_pos inp)
               (Success s1, Failure pos2) -> succeed s1 pos2
               (Failure pos1, Success s2) -> succeed s2 pos1
               (Failure pos1, Failure pos2) -> Failure $! fp
                   where fp = furthest_pos pos1 pos2
    where succeed (v, n, inp, pos) pos' = fp `seq` Success (v, n, inp, fp)
              where fp = furthest_pos pos pos'

(<*>) :: Parser a (b -> c) -> Parser a b -> Parser a c
p1 <*> p2 = \inp -> case p1 inp of
                        Failure pos1 -> Failure pos1
                        Success s1 ->
                            case p2 (rinp s1) of
                                Failure pos2 -> Failure $! fp
                                    where fp = furthest_pos (pos s1) pos2
                                Success s2 -> Success (seq_comp s1 s2)
    where seq_comp (v1, n1, _, pos1) (v2, n2, r, pos2) = fp `seq` (v1 v2, n1 + n2, r, fp)
              where fp = furthest_pos pos1 pos2

(<!>) :: Parser a b -> Parser a c -> Parser a b
p <!> q = \inp -> case (p inp, q inp) of
                      (p_res, Failure _) -> p_res
                      (f@(Failure _), _) -> f
                      (Success sp, Success sq)
                          | num sq >= num sp -> Failure (get_pos inp)
                          | otherwise -> Success sp
\end{code}
%(<*>) :: Parser a (b -> c) -> Parser a b -> Parser a c
%p1 <*> p2 = \inp -> case p1 inp of
%                        Failure pos1 -> Failure pos1
%                        Success (v1, n1, inp1, pos1) ->
%                            case p2 inp1 of
%                                Failure pos2 -> Failure $! fp
%                                    where fp = furthest_pos pos1 pos2
%                                Success (v2, n2, inp2, pos2) ->
%                                    fp `seq` Success (v1 v2, n1 + n2, inp2, fp)
%                                        where fp = furthest_pos pos1 pos2
%                                Success (v2, n2, inp2, pos2) ->
%                                    fp `seq` Success (v, n, inp2, fp)
%                                        where v = v1 v2
%                                              n = n1 + n2
%                                              fp = furthest_pos pos1 pos2
%(<!>) :: Parser a b -> Parser a c -> Parser a b
%p <!> q = \inp -> case q inp of
%                      Failure _ -> p inp
%                      Success sq ->
%                          case p inp of
%                              Failure fp -> Failure fp
%                              Success sp ->
%                                  if num sq >= num sp
%                                  then Failure (get_pos inp)
%                                  else Success sp
\caption{Second draft of the primitive parsers}
\label{fig:primitive_parsers_second_draft}
\end{figure}

Finally note that all the current parsers lose the position information
when they are applied. This means that once we have lexed the input
there is no position information for the syntax analyser and other
stages which is clearly unacceptable. We therefore define the primitive
function \hvarid{keep\_pos}, as shown in Figure~\ref{fig:remember_pos},
which takes a parser and returns a parser identical\footnote{Technically
this is not true as \hvarid{keep\_pos} can alter the strictness
properties of the parser; for example, \hvarid{pSucceed} \hvarid{v} is not strict
in its next argument, but \hvarid{keep\_pos} (\hvarid{pSucceed}
\hvarid{v}) is.} except
the return value is paired with the position of the start of the input.
We also define ${<}{\&}{>}$ and ${<}{\&}$, also shown in
Figure~\ref{fig:remember_pos}, which are equivalent to ${<}{\$}{>}$ and
${<}{\$}$ respectively except they apply \hvarid{keep\_pos} to the
resulting parser; note, however, that
\hbox{\hvarid{f} ${<}{\&}{>}$ \hvarid{p1} ${<}{*}{>}$ \hvarid{p2}} does not
make sense as you can not apply something of type
\hbox{($\alpha$, \htycon{Position})}
to a value.

\begin{figure}
\begin{code}
keep_pos :: Parser a b -> Parser a (b, Position)
keep_pos p = \inp -> case p inp of
                         Failure pos -> Failure pos
                         Success (v, n, inp', pos)
                             -> Success ((v, get_pos inp), n, inp', pos)

(<&>) :: (b -> c) -> Parser a b -> Parser a (c, Position)
f <&> p = keep_pos (f <$> p)

(<&) :: b -> Parser a c -> Parser a (b, Position)
f <& p = const f <&> p
\end{code}
\caption{Code for \hvarid{keep\_pos}, ${<}{\&}{>}$ and ${<}{\&}$}
\label{fig:remember_pos}
\end{figure}

\subsubsection{Optimisations}

While what we have done so far is sufficient, there are a couple of
obvious places where we can optimise. First consider the case of a
Choice where we know that if one of the choices succeeds the other never
will; for example, when we are looking for keywords during lexical
analysis if the parser for the keyword \hkeyword{case} succeeds then
there is no point in trying the parser for the keyword
\hkeyword{do}---not only can it not succeed but its furthest failure
point is guaranteed to be closer. We must be careful though---after
finding the keyword \hkeyword{in} we must still look for keywords like
\hkeyword{instance} and \hkeyword{infix}. Looking for \hkeyword{in} only
if we fail to find \hkeyword{instance} is fine, but note that the
furthest position reached of the first parser is relevant; for example,
the input may have been ``\verb!instanx!''.

There are many places where this is useful, both in lexical and
syntactical analysis; in particular it is often obvious where it could
be used within a production. We therefore create a new primitive
${<}{|}$, which is equivalent to ${<}{|}{>}$ except it does not try the
right parser if the left succeeds, for performance reasons. The
symmetric equivalent ${|}{>}$ is defined in terms of ${<}{|}$. Code
for both is given in Figure~\ref{fig:short_circuit_choice}.

%\begin{figure}
%\begin{code}
%(<|) :: Parser a b -> Parser a b -> Parser a b
%p1 <| p2 = \inp -> case p1 inp of
%                       Failure pos1 ->
%                           case p2 inp of
%                               Success (v2, n2, inp2, pos2) ->
%                                   fp `seq` Success (v2, n2, inp2, fp)
%                                       where fp = furthest_pos pos1 pos2
%                               Failure pos2 ->
%                                   Failure $! fp
%                                       where fp = furthest_pos pos1 pos2
%                       success -> success
%
%(|>) :: Parser a b -> Parser a b -> Parser a b
%p1 |> p2 = p2 <| p1
%\end{code}
%\caption{Code for short circuit Choice operators ${<}{|}$ and ${|}{>}$}
%\label{fig:short_circuit_choice}
%\end{figure}

\begin{figure}
\begin{code}
(<|) :: Parser a b -> Parser a b -> Parser a b
p1 <| p2 = \inp -> case (p1 inp, p2 inp) of
                       (Success r1, _) -> Success r1
                       (Failure pos1, Success (v2, n2, inp2, pos2)) ->
                               fp `seq` Success (v2, n2, inp2, fp)
                                   where fp = furthest_pos pos1 pos2
                       (Failure pos1, Failure pos2) ->
                               Failure $! fp
                                   where fp = furthest_pos pos1 pos2

(|>) :: Parser a b -> Parser a b -> Parser a b
p1 |> p2 = p2 <| p1
\end{code}
\caption{Code for short circuit Choice operators ${<}{|}$ and ${|}{>}$}
\label{fig:short_circuit_choice}
\end{figure}

Another cause of inefficiency is to be found in the \hvarid{pList}
function. Although we know that a list of at least 0 things will always
succeed we still wrap the results with \htycon{Succeed} constructors and
then unwrap them with pattern matching later. To get around this we
break up the definition of \htycon{Parser} and \htycon{Parsed} and introduce
two new primitives \hvarid{pList0} and \hvarid{pListS}. Both match a list of
at least 0 things matched by the parser given as their first argument;
the difference is that \hvarid{pListS} returns an \htycon{SParser} which in
turn will produce an \htycon{SParsed}, while \hvarid{pList0} takes this
value and wraps it up with \htycon{Success}. Only \htycon{Parser} and
\hvarid{pList0} are exported from the primitives module and we then insert
a new definition for \hvarid{pList} to make use of it. This is all shown
in Figure~\ref{fig:opt_plist}.

\begin{figure}
\begin{code}
type SParsed a b = (b, Int, [(a, Position)], Position)
data Parsed a b = Success (SParsed a b)
                | Failure Position
type SParser a b = [(a, Position)] -> SParsed a b
type Parser a b = [(a, Position)] -> Parsed a b

pList :: Int -> Parser a b -> Parser a [b]
pList 0 p = pList0 p
pList (i+1) p = (:) <$> p <*> pList i p

pList0 :: Parser a b -> Parser a [b]
pList0 p = \inp -> Success (pListS p inp)
pListS :: Parser a b -> SParser a [b]
pListS p = \inp -> case p inp of
                       Success (v1, n1, inp1, pos1) ->
                           fp `seq` (v1:v2, n1 + n2, inp2, fp)
                           where (v2, n2, inp2, pos2) = pListS p inp1
                                 fp = furthest_pos pos1 pos2
                       Failure pos -> ([], 0, inp, pos)
\end{code}
\caption{Optimised \hvarid{pList} and the new types needed}
\label{fig:opt_plist}
\end{figure}

With the previous optimisation it was clear that we would gain benefit
as it means that a lot of the time we will be able to not apply parsers.
In this case, though, there is the possibility that a compiler might
optimise out the redundant applications meaning all we are doing is
obfuscating the code. As Figure~\ref{fig:opt_plist_time} shows, though,
this optimisation is worthwhile, giving better than a 40\% speedup of
the completed pretty-printer on a 43k Haskell script.

\begin{figure}
\begin{center}
\begin{tabular}{|l|l|}
\hline
\textbf{Using normal or optimised \hvarid{pList}}&\textbf{CPU time taken}\cr
\hline
Normal&10m39.840s\cr
Optimised&6m16.180s\cr
\hline
\end{tabular}
\end{center}
\caption{Experimental data: Times for complete runs with and without
         optimised \hvarid{pList}}
\label{fig:opt_plist_time}
\end{figure}

\subsection{The lexer}

Now that we have completed our PC library, constructing the lexer is
largely a mechanical task. To save us having to remember the types of
the smaller lexers we combine to make larger lexers we shall adopt the
naming convention in
Figure~\ref{fig:lexical_analyser_naming_convention}.

\begin{figure}
\noindent
\hvarid{lexc}\_$\ast$ :: \htycon{Parser} \htycon{Char} \htycon{Char}\\
\hvarid{lexs}\_$\ast$ :: \htycon{Parser} \htycon{Char} \htycon{String}\\
\hvarid{lext}\_$\ast$ :: \htycon{Parser} \htycon{Char} [(\htycon{Token}, \htycon{Position})]\\
\hvarid{lexts}\_$\ast$ :: \htycon{Parser} \htycon{Char} (\htycon{Token}, \htycon{Position})\\
\caption{Naming convention for lexical analyser components}
\label{fig:lexical_analyser_naming_convention}
\end{figure}

Our implementation will be faithful to the report in all but one
aspect---we will not attempt to implement the productions for Unicode
characters shown in Figure~\ref{fig:unicode}, instead allowing only the
ASCII subset explicitly listed. We are in good company here---of the
Haskell implementations only the poorly supported \program{hbc} supports
Unicode, although support is beginning to appear in the more mainstream
implementations.

\begin{figure}
\begin{tabular}{lcl}
$uniWhite$&$\rightarrow$&any Unicode character defined as whitespace\cr
$uniSmall$&$\rightarrow$&any Unicode lowercase letter\cr
$uniLarge$&$\rightarrow$&any uppercase or titlecase Unicode letter\cr
$uniSymbol$&$\rightarrow$&any Unicode symbol or punctuation\cr
$uniDigit$&$\rightarrow$&any Unicode decimal digit\cr
\end{tabular}
\caption{Unicode lexical analysis productions}
\label{fig:unicode}
\end{figure}

The simpler terminal productions can be written using \hvarid{pPred} and a
predicate from the \htycon{Char} library; see
Figure~\ref{fig:lexical_pred_productions} for an example.
Figure~\ref{fig:lexical_term_productions} shows how \hvarid{pSym} (and, in
other cases, \hvarid{pSyms}) are used to match other terminals and how our
${<}{|}{>}$ operator, or one of its optimised alternatives, is analogous
to the $\mid$ symbol used to represent choice in the report. This
analogy also holds for non-terminals, as shown in
Figure~\ref{fig:lexical_nonterminal_choice}. Within each production the
report uses juxtaposition to represent sequential composition which,
together with a suitable function, is analogous to our ${<}{*}{>}$
operator as shown in Figure~\ref{fig:lex_an_seq_comp_and_exclusion}.
This figure also shows how we use \hvarid{pList} with first argument 0 for
the $\{X\}$ syntax representing any number, including 0, of $X$s and how
our ${<}{!}{>}$ operator can be used in place of the exclusion syntax
$X_{{<}Y{>}}$ used in the report. Where the report uses the $[X]$ syntax
we have a choice between using ${<}{|}{>}$ operator or `\hvarid{opt}`; we
pick one depending on which fits best in the particular circumstance.
Finally we make use of ${<}{\&}{>}$ wherever we are producing tokens to
keep the position information for the syntax analyser and other later
phases; for an example see Figure~\ref{fig:lex_ana_remember_pos}. The
type of the complete lexer is thus
\hbox{[(\htycon{Char}, \htycon{Position})] $\rightarrow$ [(\htycon{Token}, \htycon{Position})]}.

\begin{figure}
\begin{tabular}{lcl}
$ascDigit$&$\rightarrow$&$0 \mid 1 \mid \ldots \mid 9$\cr
\end{tabular}

\begin{code}
lexc_ascDigit :: Parser Char Char
lexc_ascDigit = pPred isDigit
\end{code}
\caption{Lexical analysis terminal productions implemented with predicates}
\label{fig:lexical_pred_productions}
\end{figure}

\begin{figure}
\begin{tabular}{lcl}
$\mathit{charesc}$&$\rightarrow$&$a \mid b \mid f \mid n \mid r \mid t
                                    \mid v \mid \backslash \mid "
                                    \mid \textrm{'} \mid \&$\cr
\end{tabular}

\begin{code}
lexs_charesc :: Parser Char String
lexs_charesc = wrap <$> lexc_charesc

lexc_charesc :: Parser Char Char
lexc_charesc = pSym 'a' <|  pSym 'b'  <|  pSym 'f' <|  pSym 'n'
           <|  pSym 'r' <|  pSym 't'  <|  pSym 'v' <|  pSym '\\'
           <|  pSym '"' <|  pSym '\'' <|  pSym '&'
\end{code}
\caption{Other lexical analysis terminal productions}
\label{fig:lexical_term_productions}
\end{figure}

\begin{figure}
\begin{tabular}{lcl}
$\mathit{literal}$&$\rightarrow$&$\mathit{integer} \mid \mathit{float}
                             \mid \mathit{char}    \mid \mathit{string}$\cr
\end{tabular}

\begin{code}
lext_literal :: Parser Char (Token, Position)
lext_literal = integer <|> float <|> char <|> string
\end{code}
\caption{Lexical analysis choice of non-terminals}
\label{fig:lexical_nonterminal_choice}
\end{figure}

\begin{figure}
\begin{tabular}{lcl}
$\mathit{varid}$&$\rightarrow$&$
  {\left(\mathit{small}\ \left\{ \mathit{small}
                            \mid \mathit{large}
                            \mid \mathit{digit}
                            \mid \textrm{'} \,\right\}\right)}
      _{<\mathit{reservedid}>}
  $\cr
$\mathit{conid}$&$\rightarrow$&$
  \mathit{large}\ \left\{ \mathit{small} \mid \mathit{large}
                     \mid \mathit{digit} \mid \textrm{'} \,\right\}
  $\cr
\end{tabular}

\begin{code}
id_body :: Parser Char Char
id_body = lexc_small <|  lexc_large <|  lexc_digit <|  pSym '\''
lexs_varid :: Parser Char String
lexs_varid = (:)
         <$> lexc_small
         <*> pList 0 id_body
         <!> lext_reservedid
lexs_conid :: Parser Char String
lexs_conid = (:)
         <$> lexc_large
         <*> pList 0 id_body
\end{code}
\caption{Lexical analysis sequential composition and exclusion}
\label{fig:lex_an_seq_comp_and_exclusion}
\end{figure}

\begin{figure}
\begin{tabular}{lcl}
$\mathit{opencom}$&$\rightarrow$&\texttt{\{-}\cr
\end{tabular}

\begin{code}
lexs_opencom = NCommentOpen <&  pSyms "{-"
\end{code}
\caption{Remembering the position in the lexical analyser}
\label{fig:lex_ana_remember_pos}
\end{figure}

We have some choice as to what tokens we produce. For example, there are
three ways we might handle literals:
one option is to produce a \htycon{Literal} token which contains the
information for either an integer, float, character or string; another
option is to have four seperate tokens, one for each type of literal;
the third approach is to have tokens for even the contents of strings
rather than a single token for the whole string. As we will often want
to treat the type of literal differently, for example, when parsing
infix declarations we will need to look for integers, we decide against
the first option. The last option makes it difficult to handle strings
later, so we go for the second option. We use a
second data type, \htycon{StringContents}, to describe the contents of a
string; for the other literals we make do with a \htycon{String}.
Similarly we put the contents of ordinary comments into a single token,
with the number of leading dashes as an \htycon{Int} and the rest of the
content as a \htycon{String}, but decide the easiest way to represent
nested comments is to have tokens for the beginning and end markers and
design the lexer such that only \htycon{WhiteChar}, \htycon{Newline} and
nested comment tokens appear between them.

There are a few places where we choose to do things slightly differently
to the report. Perhaps the most common is where we merge $X\{X\}$, using
just \hvarid{pList} 1; for an example see
Figure~\ref{fig:lex_ana_better_than_report_lists}. Also, the semantics
of ${<}{!}{>}$ are different to those of the $X_{{<}Y{>}}$ syntax used in
the report. Our semantics allow us to implement the \textit{ANYseq}
production more clearly, as shown in
Figure~\ref{fig:lex_ana_better_than_report_anyseq}.

\begin{figure}
\subfigure[Lists]{
\begin{tabular}{l}
\begin{tabular}{lcl}
$\mathit{decimal}$&$\rightarrow$&$\mathit{digit}\
                                \left\{ \mathit{digit} \right\}$\cr
\end{tabular}\cr
\cr
\begin{code}
lexs_decimal :: Parser Char String
lexs_decimal = pList 1 lexc_digit
\end{code}
\cr
\end{tabular}
\label{fig:lex_ana_better_than_report_lists}
}
\addtocounter{figure}{-1}

\subfigure[ANYseq]{
\begin{tabular}{l}
\begin{tabular}{lcl}
$\mathit{ANYseq}$&$\rightarrow$&$\left\{ \mathit{ANY} \right\}
    _{\left<\left\{ \mathit{ANY} \right\}\
            \left( \mathit{opencom} \mid \mathit{closecom}\right)
            \left\{ \mathit{ANY} \right\}
      \right>}$\cr
\end{tabular}\cr
\cr
\begin{code}
lexts_ANYseq :: Parser Char [(Token, Position)]
lexts_ANYseq = pList 0 (lext_ANY <!> (lexs_opencom <|  lexs_closecom))
\end{code}
\cr
\end{tabular}
\label{fig:lex_ana_better_than_report_anyseq}
}
\addtocounter{figure}{-1}
\caption{Lexical analysis: Doing better than the report}
\label{fig:lex_ana_better_than_report}
\end{figure}

\subsection{The layout rule}

\subsubsection{Indent Marking}

The report describes the layout rule as a 2 part process. The first
part, whose rules are summarised in
Figure~\ref{fig:indent-marking_rule}, can be straightforwardly turned
into a piece of code that simply walks through the list adding
additional tokens as required. We use an abstract datatype, defined in
Figure~\ref{fig:indent-marking_code} along with the functions which use
it, to remember what we expect next. The externally visible function,
\hvarid{indentmarkify}, just calls \hvarid{indentmark} with the initial
state \htycon{BOM}; this then simply implements the rules on a case by
case basis.

\begin{figure}
Given a list of lexemes, as specified by the lexical syntax, augment
the list with additional tokens as described by these rules:
\begin{itemize}
      \item
            If a \hkeyword{let}, \hkeyword{where}, \hkeyword{do}, or \hkeyword{of} keyword is not
            followed by the lexeme ``\{'', the token ``\{$n$\}'' is
            inserted after the keyword, where $n$ is the indentation of
            the next lexeme if there is one, or 0 if the end of file has
            been reached.
      \item If the first lexeme of a module is not ``\{'' or \hkeyword{module},
            then it is preceded by ``\{$n$\}'' where $n$ is the indentation
            of the lexeme.
      \item Where the start of a lexeme does not follow a complete
            lexeme on the same line, this lexeme is preceded by
            ``${<}n{>}$''
            where $n$ is the indentation of the lexeme, provided that it
            is not, as a consequence of the first two rules, preceded by
            ``\{$n$\}''.
\end{itemize}
   
The following rules are also in effect while this is being done:
\begin{itemize}
    \item The ``newline'' lexeme starts a new line.
    \item The first column is designated column 1, not 0.
    \item Tab stops are 8 characters apart.
    \item A tab character causes the insertion of enough spaces to align
          the current position with the next tab stop.
    \item Unicode characters are considered to be of the same, fixed,
          width as an ASCII character.
\end{itemize}
\caption{The indent-marking rule}
\label{fig:indent-marking_rule}
\end{figure}

\begin{figure}
\begin{code}
data IndentState = BOL
                 | BOS
                 | BOM
                 | Normal

indentmarkify :: [(Token, Position)] -> [(Token, Position)]
indentmarkify = indentmark BOM

indentmark :: IndentState -> [(Token, Position)] -> [(Token, Position)]
indentmark BOM [] = [(IndentToken 0, end_of_file)]
indentmark BOS [] = [(IndentToken 0, end_of_file)]
indentmark is [] = []
indentmark Normal (tp@(Newline, _):tps) = tp:indentmark BOL tps
indentmark is (tp@(t, _):tps)
 | is_white_space t = tp:indentmark is tps
indentmark BOM tps@(tp@(t, p):tps')
 | t == Special '{'         = tp:indentmark Normal tps'
 | t == ReservedID "module" = tp:indentmark Normal tps'
 | otherwise                = this:indentmark Normal tps
  where this = (IndentToken (get_pos_char p), p)
indentmark BOS tps@(tp@(t, p):tps')
 | t == Special '{' = tp:indentmark Normal tps'
 | otherwise        = this:indentmark Normal tps
  where this = (IndentToken (get_pos_char p), p)
indentmark BOL tps@((_, p):_)
 | otherwise        = this:indentmark Normal tps
  where this = (IndentLine (get_pos_char p), p)
indentmark _ (tp@(ReservedID r, _):tps)
 | r `elem` ["let", "where", "do", "of"] = tp:indentmark BOS tps
indentmark Normal (tp:tps) = tp:indentmark Normal tps
\end{code}
\caption{The indent-marking code}
\label{fig:indent-marking_code}
\end{figure}

\subsubsection{The layout rule}

Having augmented the list of tokens we apply the function given in
pseudo-code in the report, which we have reproduced in
Figure~\ref{fig:layoutrule} for your convenience. The function is called
with \hvarid{L} \hvarid{tokens} [\,] where \hvarid{tokens} is the
augmented token stream generated above. While most of the function could
be implemented in a couple of dozen lines of code there is one clause,
pointed at by a {\font\dingbat=bbding10\dingbat\symbol{'021}} in
Figure~\ref{fig:layoutrule}, that is significantly harder.

\begin{figure}
\begin{tabular}{lcl>{\quad}l}
\hvarid{L} (${<}n{>}$:\hvarid{ts}) (\hvarid{m}:\hvarid{ms})&=&`;':(\hvarid{L} \hvarid{ts} (\hvarid{m}:\hvarid{ms}))&\hkeyword{if} $m = n$\cr
                 &=&`\}':(\hvarid{L} (${<}n{>}$:\hvarid{ts}) \hvarid{ms})&\hkeyword{if} $n < m$\cr
\hvarid{L} (${<}n{>}$:\hvarid{ts}) \hvarid{ms}&=&\hvarid{L} \hvarid{ts} \hvarid{ms}\cr
\hvarid{L} (\{\hvarid{n}\}:\hvarid{ts}) (\hvarid{m}:\hvarid{ms})&=&`\{':(\hvarid{L} \hvarid{ts} (\hvarid{n}:\hvarid{m}:\hvarid{ms}))&\hkeyword{if} $n > m$\cr
\hvarid{L} (\{\hvarid{n}\}:\hvarid{ts}) [\,]&=&`\{':(\hvarid{L} \hvarid{ts} [\hvarid{n}])&\hkeyword{if} $n > 0$\cr
\hvarid{L} (\{\hvarid{n}\}:\hvarid{ts}) \hvarid{ms}&=&`\{':`\}':(\hvarid{L} (${<}n{>}$:\hvarid{ts}) \hvarid{ms})\cr
\hvarid{L} (`\}':\hvarid{ts}) (0:\hvarid{ms})&=&`\}':(\hvarid{L} \hvarid{ts} \hvarid{ms})\cr
\hvarid{L} (`\}':\hvarid{ts}) \hvarid{ms}&=&\hvarid{parse-error}\cr
\hvarid{L} (`\{':\hvarid{ts}) \hvarid{ms}&=&`\{':(\hvarid{L} \hvarid{ts} (0:\hvarid{ms}))\cr
\hbox to 0pt{\hss{\font\dingbat=bbding10\dingbat\symbol{'021}}\quad}\hvarid{L}
                            (\hvarid{t}:\hvarid{ts}) (\hvarid{m}:\hvarid{ms})&=&`\}':(\hvarid{L} (\hvarid{t}:\hvarid{ts}) \hvarid{ms})&\hkeyword{if}
                            $m \ne 0$ and \hvarid{parse-error}(\hvarid{t})\cr
\hvarid{L} (\hvarid{t}:\hvarid{ts}) \hvarid{ms}&=&t:(\hvarid{L} \hvarid{ts} \hvarid{ms})\cr
\hvarid{L} [\,] [\,]&=&[\,]\cr
\hvarid{L} [\,] (\hvarid{m}:\hvarid{ms})&=&`\}':\hvarid{L} [\,] \hvarid{ms}&\hkeyword{if} $m \ne 0$\cr
\end{tabular}
\caption{The layout rule}
\label{fig:layoutrule}
\end{figure}

\begin{figure}
\begin{code}
init_states :: Automaton_State
step :: Automaton_State -> Token -> Automaton_State
no_such_state :: Automaton_State -> Bool
\end{code}
\caption{Automaton exported type signatures}
\label{fig:ia-type-sigs}
\end{figure}

In order to know if a parse error would occur we will use a state
machine; for now we will assume the existence of such an automaton which
exports the type \htycon{Automaton\_State} and 3 functions, whose type
signatures are given in Figure~\ref{fig:ia-type-sigs};
\hvarid{init\_states} is the initial state of the automaton at the
beginning of a script, \hvarid{step} takes the current state of the
automaton, the next token and returns the new state of the automaton and
\hvarid{no\_such\_state} takes an automaton state and returns true iff that
is not a valid state, i.e., the tokens the automaton has been stepped
with thus far to get to the state given do not represent a valid prefix
of a Haskell script.

The mutually recursive functions \hvarid{step} and \hvarid{step'} apply the
rules of the layout rule and actually step the automaton, checking a
valid state is reached, respectively. The helper function
\hvarid{parse\_error} returns \htycon{True} iff the next token would cause a
parse error and \hvarid{im} serves simply to make the code more readable.

\begin{figure}
\begin{code}
offsideify :: [(Token, Position)] -> [(Token, Position)]
offsideify = step init_states []

step :: Automaton_State -> [Int] -> [(Token, Position)] -> [(Token, Position)]
step as ms (tp@(t, _):tps)
 | is_white_space t = tp:step as ms tps
step as (m:ms) tps@((IndentLine n, p):tps')
 | m == n = step' as (m:ms) [im ';' p] tps'
 | n < m  = step' as    ms  [im '}' p] tps
step as ms ((IndentLine _, _):tps) = step as ms tps
step as ms@(m:_) ((IndentToken n, p):tps)
 | n > m = step' as (n:ms) [im '{' p] tps
step as [] ((IndentToken n, p):tps)
 | n > 0 = step' as [n] [im '{' p] tps
step as ms ((IndentToken n, p):tps)
 = step' as ms [im '{' p, im '}' p] ((IndentLine n, p):tps)
step as (0:ms) (tp@(Special '}', _):tps) = step' as ms [tp] tps
step as ms (tp@(Special '{', _):tps) = step' as (0:ms) [tp] tps
step as (m:ms) tps@((t, p):_)
 | m /= 0 && parse_error as t = step' as ms [im '}' p] tps
step as ms (tp:tps) = step' as ms [tp] tps
step _  [] [] = []
step as (m:ms) []
 | m /= 0 = step' as ms [im '}' end_of_file] []
step _ (0:_) [] = error ("Missing } at " ++ show_pos end_of_file)

step' :: Automaton_State
      -> [Int]
      -> [(Token, Position)]
      -> [(Token, Position)]
      -> [(Token, Position)]
step' as ms [] tps = step as ms tps
step' as ms (tp@(t, p):tps) tps'
 | no_such_state as' = error ("No such state at " ++ show_pos p)
 | otherwise         = tp:step' as' ms tps tps'
  where as' = step_automaton as t

parse_error :: Automaton_State -> Token -> Bool
parse_error _ (Special '}') = True
parse_error ss t = no_such_state (step ss t)

im :: Char -> Position -> (Token, Position)
im c p = (ImplicitSpecial c, p)
\end{code}
\caption{The layout rule code}
\label{fig:layoutrule_code}
\end{figure}

\subsubsection{The automaton}

All that remains is to implement the automaton. There is a large amount
of literature on the creation of automata, e.g.,~\cite{dragonbook}, but
we will settle here for a simple implementation which nevertheless takes
linear time with the state transition table which we will provide it.
This state transition table is specified by a text file using a BNF-like
grammar. Additional syntax is used as we need to be able to convey more
information than the report, which is merely informative.

%described in Figure~\ref{fig:automaton_bnf_description}.
%
%\begin{figure}
%\begin{tabular}{ll}
%"case" == &bar\cr
%foo&bar\cr
%\end{tabular}
%\caption{Description of the BNF-like grammar used to specify the state
%transition table of the automaton}
%\label{fig:automaton_bnf_description}
%\end{figure}

The actual table description
%, given in
%Appendix~\ref{app:automaton_description},
is based on the context free
grammar in the Haskell report. As we are ignoring the fixity information
at this stage there are a couple of places where we can significantly
simply things, e.g., the grammar for expressions, shown in
Figure~\ref{fig:expi_lexical_syntax}, is quite complex, but this
simplification means we can reduce it to that shown in
Figure~\ref{fig:expi_simplified_lexical_syntax}.

\begin{figure}
\subfigure[Grammar in Haskell~98 report]{
\begin{tabular}{lcl}
$\mathit{exp}^i$&$\rightarrow$&$\mathit{exp}^{i+1}
                     \left[ \mathit{qop}^{\left(n,i\right)}
                            \mathit{exp}^{i+1} \right]$\cr
&$\mid$&$\mathit{lexp}^i$\cr
&$\mid$&$\mathit{rexp}^i$\cr
$\mathit{lexp}^i$&$\rightarrow$&$\left(\mathit{lexp}^i \mid
                                       \mathit{exp}^{i+1}\right)
                                 \mathit{qop}^{\left(l,i\right)}
                                 \mathit{exp}^{i+1}$\cr
$\mathit{lexp}^6$&$\rightarrow$&$-\mathit{lexp}^7$\cr
$\mathit{lexp}^i$&$\rightarrow$&$\mathit{exp}^{i+1}
                                 \mathit{qop}^{\left(r,i\right)}
                                 \left(\mathit{rexp}^i \mid
                                       \mathit{exp}^{i+1}\right)$\cr
$\mathit{exp}^{10}$&$\rightarrow$&\dots\cr
\end{tabular}
\label{fig:expi_lexical_syntax}
}

\subfigure[Simplified grammar]{
\begin{tabular}{lcl}
$\mathit{exp}^i$&$\rightarrow$&$ \left[ - \right] \mathit{exp}^{10}
                             \left[ \mathit{qop} \, \mathit{exp}^i \right]$\cr
$\mathit{exp}^{10}$&$\rightarrow$&\dots\cr
\end{tabular}
\label{fig:expi_simplified_lexical_syntax}
}
\caption{Context free grammar for expressions}
\end{figure}

While it would be possible to make only these changes we also choose to
eliminate left-recursion and left-factor within each production; this is
sufficient to remove all infinite loops and exponential growth from the
grammar. Note that this does not make the resulting state machine
deterministic as the left-factoring is only local; there are still
states from which an opening parenthesis, for example, could move to one
of a number of states.

We will think of our automaton as being composed of numerous smaller
automata which can link to each other. Each of these automata
corresponds to the production of a non-terminal in the context free
grammar. When a link between automata is followed a return state within
the calling automaton is also given. When we reach the end state of the
called automaton we return to the return state in the calling automaton.
Our state is therefore a stack of \htycon{LocalState}s, or pairs of
integers, identifying a production and position within the production.
The top pair on the stack is our current position and the rest of the
stack is the history of return addresses. These datastructures are shown
in Figure~\ref{fig:automaton_data_structures}.

\begin{figure}
\begin{code}
type Automaton_State = [State]
type State = [LocalState]
type LocalState = (Integer, Integer)
\end{code}
\caption{Automaton data structures}
\label{fig:automaton_data_structures}
\end{figure}

The first function our automaton must provide is an inital state.
Initially we have a single state at the start of the first production
and we have no return address. We will guarantee the first production is
given number 0 and that the initial state of each automaton is state 0,
so this is just $\left[\left[\left(0, 0\right)\right]\right]$ as shown
in Figure~\ref{fig:automaton_code}.

\begin{figure}
\begin{code}
init_states :: Automaton_State
init_states = [[( 0,  0)]]

no_such_state :: Automaton_State -> Bool
no_such_state = null

step :: Automaton_State -> Token -> Automaton_State
step ss t = [ls | (e, ls) <- closure ss, can_step e (un_im t)]

un_im :: Token -> Token
un_im (ImplicitSpecial s) = Special s
un_im t = t

can_step :: Either Token Func -> Token -> Bool
can_step (Left s) t = s == t
can_step (Right (Func (_, f))) t = f t

closure :: [State] -> [(Either Token Func, State)]
closure = remove_dupes . sort . do_all_nulls

remove_dupes :: (Eq a) => [a] -> [a]
remove_dupes (a:b:xs)
 | a == b    =   remove_dupes (b:xs)
 | otherwise = a:remove_dupes (b:xs)
remove_dupes xs = xs

do_all_nulls :: [State] -> [(Either Token Func, State)]
do_all_nulls [] = []
do_all_nulls ([]:qs) = do_all_nulls qs
do_all_nulls ((qh:qt):qs)
 = case lookupFM state_changes qh of
       Nothing -> error "do_all_nulls: Can't happen (Nothing)"
       Just x -> case x of
            Jump xs -> do_all_nulls (tailed_xs ++ qs)
                where tailed_xs = map ((: qt) . (,) h) xs
            Long_Jump i j -> do_all_nulls (((i, 0):(h, j):qt):qs)
            SLiteral e i -> (e, (h, i):qt):do_all_nulls qs
            End -> do_all_nulls (qt:qs)
  where h = fst qh

state_changes :: FiniteMap LocalState StateChange
state_changes = listToFM state_changes'

state_changes' :: [(LocalState, StateChange)]
\end{code}
\caption{Automaton code}
\label{fig:automaton_code}
\end{figure}

Next we must be able to step the automaton, i.e., given a \htycon{Token}
$t$ and the current automaton state we must return the new automaton
state. To do this we take each of the set of \htycon{State}s our automaton
could be in and try to step them with $t$.

The state transition table defines a partial function from
\htycon{LocalState}s to \htycon{StateChange}s, a data structure which
describes a change of state. Equivalently,
$\hvarid{stt} \colon \htycon{LocalState} \rightharpoonup \htycon{StateChange}$.
The data structure used by the automaton to represent state changes is
shown in Figure~\ref{fig:state_change_data_structures} and will be
explained shortly.

\begin{figure}
\begin{code}
data StateChange = Jump [Integer]
                 | Long_Jump Integer Integer
                 | SLiteral (Either Token (Token -> Bool)) Integer
                 | End
  deriving (Eq, Ord)
\end{code}
\caption{State change data structures}
\label{fig:state_change_data_structures}
\end{figure}

We can break down the possible state changes into two sets: those which
are dependent on the token stream and those which are not. To try to
step a \htycon{State} we first apply apply all state changes that are
independant of the token stream.

Suppose the top of a \htycon{State} stack $s$ is $\left(h, i\right)$,
i.e., we are in position $i$ within production $h$, the remainder of the
stack is $s'$ and let $sc = stt \cdot \left(h, i\right)$. There are 3
patterns of $sc$ that correspond to token independent state changes:

\begin{iglist}
\item[\htycon{End}]{
    This indicates that we have reached the end of this
    production. The state is thus equivalent to $s'$.
}
\vspace{-\smallskipamount}
\item[\htycon{Jump} $js$]{
    This represents a choice to jump to any state $j \in js$. This state
    is equivalent to the set of states
    $\left\{ \left(h, j\right):s' \mid j \in js \right\}$.
}
\vspace{-\smallskipamount}
\item[\htycon{Long\_Jump} $j$ $k$]{
    This represents a jump to another production. We jump to the start
    state of production $j$ and return to state $k$ in the current
    production, so this state is equivalent to
    $\left(j, 0\right):\left(h, k\right):s'$.
}
\end{iglist}

Although we have described equivalences it is only useful to follow the
state changes in the forward direction. We apply these rules to each of
our possible \htycon{State}s until it is not possible to apply them to any
\htycon{State}; we also remove any empty stacks as these represent
completed traversals of the whole state machine so cannot accept any
token $t$. The function \hvarid{closure}, shown in
Figure~\ref{fig:automaton_code}, implements this, using
\hvarid{do\_all\_nulls} to do the work and then removing any duplicates in
the list of states reached. It returns each of the \htycon{State}s as a
tuple of the token-dependent test and the \htycon{State} to use if the
test succeeds, i.e., if the token would be accepted at the state.

The reason for not using the \hvarid{nub} function from the \htycon{List}
standard library for removing duplicates is that profiling revealed that
this was by far the slowest part of the program; this was then replaced
as shown with the \hvarid{sort} function from the standard library but was
still responsible for the majority of execution time, so a sorting
function using merge sort was written instead. This took the time for
the complete pretty-printer to run from 19 minutes, then down to 6
minutes and finally down to 1 minute for a test input.

%My sort on time1:
%
%real    1m21.449s
%user    1m20.500s
%sys     0m0.140s
%
%GHC sort on time1:
%
%real    6m34.011s
%user    6m28.420s
%sys     0m0.150s
%
%nub on time1:
%
%real    22m4.614s
%user    19m15.900s
%sys     0m3.910s

The next thing we must do is to decide whether the token \hvarid{t} is
permitted at each of our potential states. There are two tests which we
may have to perform:

\begin{tabular}{ll}
\htycon{Left} \hvarid{u}&This succeeds iff \hvarid{t} $=$ \hvarid{u}.\cr
\htycon{Right} $\left(\htycon{SLiteral}\ \hvarid{f}\right)$&This succeeds iff \hvarid{f} \hvarid{t} holds.\cr
\end{tabular}

%\begin{iglist}
%\item[$\left(\htycon{Left}\ \hvarid{u}\right)$]{
%    This succeeds iff \hvarid{t} $=$ \hvarid{u}.
%}
%\item[\htycon{Right} $\left(\htycon{SLiteral}\ \hvarid{f}\right)$]{
%    This succeeds iff \hvarid{f} \hvarid{t} holds.
%}
%\end{iglist}

\noindent This is implemented by \hvarid{can\_step} in
Figure~\ref{fig:automaton_code}.

The stepping function, \hvarid{step}, returns each possible state just
after the point where the token is accepted. This means that even when
we have reached the end of the state machine there will be a non-empty
stack which will only become empty, and thus removed, when
\hvarid{do\_all\_nulls} is applied. As neither \hvarid{init\_states} nor
\hvarid{step} apply this to what they return the user of the automaton
is returned an empty list if and only if the sequence of tokens stepped
so far is not a valid prefix of the language. We can therefore use the
standard function \hvarid{null} for \hvarid{no\_such\_state}. The code for
this is also given in Figure~\ref{fig:automaton_code}.

The actual table has not been included due to its size. Another
program, using the PC library we have developed, parses the textual
description of the automaton% in Appendix~\ref{app:automaton_description}
and produces either the diagrams in Appendix~\ref{app:automaton_diagrams}
or the code for the automaton module.

\subsection{The fixity annotation}

Before we can lexically analyse the token stream we need to add the
fixity information so that we can parse the expressions correctly. The
details are messy and uninteresting, so we will present only a brief
overview here.

For each block we do two passes. In the first pass we walk through,
counting our nesting depth to know when we have finished the block, and
extract the top-level fixity information---more on that later. Then, in
the second pass, we change the fixities of any appropriate tokens,
dealing with sub-blocks recursively. There are various things we must
take care of such as overriding our fixity list with that of the
\hkeyword{where} clause of a declaration where appropriate; although
these are all fairly simple, a reasonable amount of code is needed to
cover them all. Finally we return the altered block of tokens together
with the set of top-level fixity information we collected in the first
pass; it is this that is of relevance if this block is a
\hkeyword{where} clause.

To extract the fixity information we not only need to look for and
extract the information from fixity declarations started with an
\hkeyword{infix}, \hkeyword{infixl} or \hkeyword{infixr} token but we
also need to notice when new functions are defined and thus override the
fixity information from a higher scope. The variety of ways in which
functions can be defined adds to the volume of code needed to handle all
cases correctly.

The \htycon{Read} instance of \htycon{Integer} makes it easy for us to
extract the value of the optional integer given in a fixity declaration.
We use \hvarid{reads} rather than \hvarid{read} so we can give a useful
error message, including position, should a bug in the lexer allow an
invalid integer through.

A complete implementation would also look for imported modules and check
what these modules export and what restrictions are placed by the import
statement. Here we settle for simply starting with the fixity
information set by the standard prelude.

This is, in the worst case, quadratic in the length of the input as, for
example, adding an extra layer to deeply nested \hkeyword{where} blocks
will require traversing all the existing input extra times to handle the
outer-most case. In practise the depth of nesting rarely goes above
three or four and so this algorithm can be thought of as linear. For
better worst-case performance a data structure which allows a token
stream to be represented as a tree of blocks could be used, but we
believe it to be overkill for real-world use.

\subsection{The syntax analyser}

The syntax analyser represents a similar challenge to the lexical
analyser and the same equivalences of our operators to the report syntax
hold. The exclusion syntax $X_{{<}Y{>}}$, for which the semantics differ
to our ${<}{!}{>}$ operator, appears only once, but we need to handle
this case specially. The relevant productions are shown in
Figure~\ref{fig:cfg_exclusion_cfg}; although the report allows a
labelled update to be applied to other productions of \textit{aexp} such
as tuple this is not actually possible so for simplicity we do not allow
these cases. We use \hvarid{pMaybe} to allow an optional labelled update
after these productions; see the code for \hvarid{qvar} in
Figure~\ref{fig:cfg_exclusion_code} for details.

\begin{figure}
\subfigure[Context free grammar]{
\begin{tabular}{lcl}
$\mathit{ANYseq}$&$\rightarrow$&$\mathit{qvar}$\cr
&$\mid$&\dots\cr
&$\mid$&$\mathit{qcon}\
    \left\{ \mathit{fbind}_1, \ldots, \mathit{fbind}_n \right\}$\cr
&$\mid$&$\mathit{aexp}_{\left<\mathit{qcon}\right>}\
    \left\{ \mathit{fbind}_1, \ldots, \mathit{fbind}_n \right\}$\cr
\end{tabular}
\label{fig:cfg_exclusion_cfg}
}

\subfigure[Code]{
\begin{tabular}{l}
\begin{code}
p_aexp_qvar :: Parser Token Aexp
p_aexp_qvar = (\a m_u -> case m_u of
                             Nothing -> a
                             Just u -> Aexp_update a u)
          <$> (Aexp_qvar <$> p_qvar)
          <*> pMaybe p_aexp_update
\end{code}
\cr
\cr
\begin{code}
p_aexp_update :: Parser Token [[Fbind]]
p_aexp_update = pList 1 p_aexp_update_seg
\end{code}
\cr
\cr
\begin{code}
p_aexp_update_seg :: Parser Token [Fbind]
p_aexp_update_seg = id
                <$  p_curly_open
                <*> pList 1 p_fbind
                <*  p_curly_close
\end{code}
\cr
\end{tabular}
\label{fig:cfg_exclusion_code}
}
\addtocounter{figure}{-1}
\caption{Exclusion in the context free grammar}
\label{fig:cfg_exclusion}
\end{figure}

%stopzone

This is not the only place where we deviate from the report. The
production for \textit{body} in the report is shown in
Figure~\ref{fig:cfg_body}; the problem is that \textit{impdecls} will
consume trailing semicolons as it allows empty import declarations so
enforcing the presence of a separating semicolon if there are also
\textit{topdecls} is tricky. While we could solve this it would add a
significant amount of complexity to the code; by contrast, doing it in
another pass, which is necessary\footnote{We do not implement this final
pass as it would serve only to restrict the set of valid inputs and so
is not required for our purposes.} anyway to check other side-conditions
imposed by the report, is easy.

\begin{figure}
\begin{tabular}{lcl}
$\mathit{body}$&$\rightarrow$&
    $\left\{ \mathit{impdecls} ; \mathit{topdecls} \right\}$\cr
&$\mid$&$\left\{ \mathit{impdecls} \right\}$\cr
&$\mid$&$\left\{ \mathit{topdecls} \right\}$\cr
\end{tabular}
\caption{\textit{body} context free grammar}
\label{fig:cfg_body}
\end{figure}

Enforcing the restriction on left sections, whose production is shown in
Figure~\ref{fig:cfg_left_section}, that the operator be of a lower
precedence-level than the expression can be done, but the easier ways of
doing it are slow, as they require the expression being parsed multiple
times, and the fast ways again add complexity. Thus we also leave this
check until the next pass, allowing any expression followed by a
left-associative operator in parentheses as a left section.

\begin{figure}
\begin{tabular}{lcl}
$\mathit{aexp}$&$\rightarrow$&
    $\left( \mathit{exp}^{i+1} \mathit{qop}^{\left(l, i\right)} \right)$\cr
\end{tabular}
\caption{\textit{aexp} left section context free grammar}
\label{fig:cfg_left_section}
\end{figure}

Another issue we must deal with is left recursion. This was not an issue
in the lexical syntax, but the context free grammar has productions such
as that shown in Figure~\ref{fig:cfg_btype} where a \textit{btype} can
begin with a \textit{btype} so a direct translation into parser
combinators would yield a parser which recursed forever looking for
\textit{btype}s. However we can see that a \textit{btype} is just a list of
\textit{atype}s, giving us the code shown in
Figure~\ref{fig:cfg_btype_code}. In general, if we have a production
\hbox{$\alpha \rightarrow \alpha \beta_1 \mid \ldots \mid \alpha \beta_n
               \mid \gamma_1 \mid \ldots \mid \gamma_m$}, then this is
equivalent to the non-left recursive
\hbox{$\alpha \rightarrow \left( \gamma_1 \mid \ldots \mid \gamma_m \right)
                    \left\{ \beta_1 \mid \ldots \mid \beta_n \right\}$}. In
this case \hbox{$m = 1$}, \hbox{$n = 1$}, \hbox{$\beta_1 = \mathit{atype}$} and
\hbox{$\gamma_1 = \epsilon$}.

\begin{figure}
\subfigure[Production]{
\begin{tabular}{lcl}
$\mathit{btype}$&$\rightarrow$&
    $\left[ \mathit{btype} \right] \mathit{atype}$\cr
\end{tabular}
\label{fig:cfg_btype}
}

\subfigure[Code]{
\begin{code}
p_btype :: Parser Token Btype
p_btype = Btype <$> pList 1 p_atype
\end{code}
\label{fig:cfg_btype_code}
}
\addtocounter{figure}{-1}
\caption{\textit{btype} production and code}
\end{figure}

%stopzone

There are some cases where we implement a slightly different, but
equivalent, grammar to the report. For example, if we were to implement
the production for \textit{exp} from the report, as shown in in
Figure~\ref{fig:cfg_exp}, then all the expressions would be parsed
twice---once for each alternative. We instead implement it as if it was
the production in Figure~\ref{fig:cfg_exp_opt}. This is especially
significant as \textit{exp}s can contain \textit{exp}s so the direct
translation would have exponential time complexity as opposed to the
linear time complexity of this approach. There are more complex
examples where left-factoring is needed but the same principles can be
applied.

\begin{figure}
\subfigure[Report]{
\begin{tabular}{lcl}
$\mathit{exp}$&$\rightarrow$&$
    \mathit{exp0} :: \left[ \mathit{context}\ {=}{>} \right] \mathit{type}$\cr
&$\mid$&$\mathit{exp0}$\cr
\end{tabular}
\label{fig:cfg_exp}
}

\subfigure[Optimised]{
\begin{tabular}{lcl}
$\mathit{exp}$&$\rightarrow$&$
    \mathit{exp0} \left[ :: \left[ \mathit{context}\ {=}{>} \right]
                         \mathit{type} \right]$\cr
\end{tabular}
\label{fig:cfg_exp_opt}
}
\caption{Productions for \textit{exp}}
\end{figure}

\subsection{Coping with multiple code blocks}\label{sec:code_blocks}

Up until now we have been extracting all the code from the script and
treating it as a single block of code. However, in reality a literate
script consists of alternating blocks of explanatory text and Haskell
code. We therefore introduce the type
\begin{code}
type Input a = [Either String (String, a, Position)]
\end{code}
\noindent to hold the input. A \hbox{\htycon{Left} \hvarid{s}} represents a text block
while a \hbox{\htycon{Right} (\hvarid{s}, \hvarid{x}, \hvarid{p})} represents a code segment. Depending on
the stage of the process we are at, the type of \hvarid{x} may be \hbox{[(\htycon{Char}, \htycon{Position})]},
\hbox{[(\htycon{Token}, \htycon{Position})]} or \hbox{[\htycon{Parsed}]}.
The original text of the code block is stored
in \hvarid{s}; if at some phase an error is encountered then we will
replace this block with \hbox{\htycon{Left} \hvarid{s}}, giving a warning that we are
doing so, and continue the phase at the next code block with the state
as it was at the beginning of this block. The position of the end of
this block is stored in \hvarid{p}; we use this rather than
\hvarid{end\_of\_file} when we need to give a position and have consumed all
of the input in this block.

The first phase, unlitting and adding position information, involves a
reasonable amount of code for something that is conceptually quite
simple, so we do not give the code here. There are three stages to it:
first we read and categorise each line as either starting with a bird
track, being part of a \verb!\begin{code}!\dots\verb!\end{code}! block
or being a comment line; we add a position to each character of code. In
the second stage we walk through the list checking that all bird track
lines are not next to comment lines with any non-whitespace characters.
In the final stage we merge the consecutive lines of the same type into
blocks.

For the next phase, the lexer, we have to place some restrictions on
where we allow code blocks to end. The report allows them to end
anywhere a new line is allowed, but it is hard to know how to typeset
functions with documentation sections in the middle. We therefore
require that top-level functions are entirely contained in a single code
block. We also require that nested comments are entirely contained in a
single code block; this makes the implementation of the lexer slightly
easier at little or no inconvenience to the user. If it proves to be a
problem then it would be fairly straightforward to modify the lexer to
carry the nesting depth forward from section to section. The lexer then
becomes trivial to adapt---we simply apply the existing function to each
of the code sections in turn, changing any code section that fails to a
comment section.

The indent marking and layout rule phases are also fairly
straightforward; while indent marking, when we reach the the end of any
code block but the last we stop when our stack of indents reaches a
singleton list rather than the empty list so as to not close the
outermost braces around all the code. The actual layout rule needs a
couple of small tweaks; for example, implicit semicolons at the
beginning of code blocks are removed as they cause ugly space at the
beginning of all but the first code block.

For the syntactical analysing phase we replace our top-level
\hvarid{syn\_an} function with 4 functions: \hvarid{syn\_an\_modulehead},
\hvarid{syn\_an\_impdecls}, \hvarid{syn\_an\_topdecls} and
\hvarid{syn\_an\_moduletail}. Our initial state is a list of these 4 parsers
tupled with a boolean indicating whether they are allowed to multiple,
or indeed zero, times; i.e. it is true for only the middle two. While a
code block is non-empty we try to apply the first parser in the list; if
the list is empty then we give an error. If the parse succeeds and the
boolean is false then we remove the parser from the list and recurse
with the input remaining after the parse; if the boolean is true then we
leave the parser on the list and recurse. If the parse fails and the
boolean is true then this is acceptable so we just remove the parser
from the list and recurse; if the boolean is false then we do not allow
this parser to fail so we abort with an error.

\section{Pretty-printing Haskell in {\LaTeX}}\label{sec:pp}

Now that we have an abstract syntax tree for Haskell we need to
pretty-print it in {\LaTeX}. Rather than embedding the formatting
information in the output, with the additional disadvantage of needing
to alter the code to change it, we create a style file that defines
{\LaTeX} functions and output {\LaTeX} code that calls these functions.
A detailed description of all the {\LaTeX} functions is beyond the scope
of this report, but we give a brief overview of one of them here.

Figure~\ref{fig:style} starts by defining \verb!@layoutinner! and
\verb!@layout!, two are internal commands used to set the formatting of
paragraphs in many places. A column type, called \verb!Y!, is then
defined that is the same as a column of type \verb!X!, defined in the
\verb!tabularx! package, but with these layout rules applied. We then
have two exported functions; the first, \verb!HtLkeyword!, takes a
single argument, presumed to be a Haskell keyword, and typesets it in
bold face. The ``\verb!HtL!'' prefix stands for ``Haskell To {\LaTeX}'' and is
designed to try to avoid name collisions. The final function defined is
used to typeset \hkeyword{where} clauses. The first of the three arguments is 0
if braces are not required around the contents of the block or 1 if they
are, the second is the contents of the the block and the third is 1 if
and only if a semicolon should follow the block. The layout is handled
by a \environment{tabularx}, a type of table which will resize columns of type
\verb!X!, or \verb!Y! as it is defined in terms of \verb!X!, to make the
table the maximum width available; this allows reasonable layouts to be
easily achieved.

\begin{figure}
\verbatiminput{style.inc}
\caption{Extract from the {\LaTeX} style file}
\label{fig:style}
\end{figure}

We then write a Haskell module which works through an abstract syntax
tree outputting the appropriate {\LaTeX}, making use of these functions.
The module makes use of a few Haskell functions too, for example: a
function for pretty-printing the unqualified half of operators checks to
see if the operator is one of a list of special operators like \verb!>=!
and, if so, outputs special {\LaTeX} for it---in this case \verb!$\ge$!;
the escaping of characters necessary in {\LaTeX} is done by
\verb!escape!, which is applied to any user-entered strings that are to
be output and performs substitutions like \verb!$\backslash$! for
\verb!\!; another function substitutes Greek letters for type variables.

\section{Discussion}\label{sec:discussion}

\subsection{Evaluation}

Now that we have completed the pretty-printer and believe that each
phase works in linear time, at least during realistic use, it is time to
test our belief. The result of running the pretty-printer on input of
various lengths is shown in Figure~\ref{fig:times_speeds}; while the
results are very respectable they are not linear. A bit of
experimentation shows that the problem is the memory allocation done by
the run-time system---if we for the run-time system to use a 300MB heap
in all cases by adding \verb!+RTS -M300M -H300M! to the command line
then we get the times in Figure~\ref{fig:times_speeds_fixed_heap}.
Although these appear to show the pretty-printer working in better than
linear time this is clearly not actually possible---the cause is the
constant startup overhead. The pretty-printer is working at more than
$\frac{1}{2}\mathrm{kB}/\mathrm{s}$ on a 733MHz machine, which is fairly
modest by today's standards. The files being tested are almost entirely
Haskell code---in practise we would expect a literate script to have a
high proportion of documentation text which would be dealt with much
more quickly, so we would expect real-world usage to be even faster.

\begin{figure}
\begin{center}
\begin{tabular}{|l|l|l|}
\hline
\textbf{Source size}&\textbf{CPU time taken}&\textbf{Processing speed}\cr
\hline
44k&1m20.600s&551 bytes per second\cr
88k&2m43.620s&543 bytes per second\cr
176k&5m31.870s&535 bytes per second\cr
352k&11m14.850s&526 bytes per second\cr
704k&23m24.520s&505 bytes per second\cr
\hline
\end{tabular}
\end{center}
\caption{Experimental data: Times and speeds for complete runs on
         various input sizes with the heap controlled by the run-time system}
\label{fig:times_speeds}
\end{figure}

\begin{figure}
\begin{center}
\begin{tabular}{|l|l|l|}
\hline
\textbf{Source size}&\textbf{CPU time taken}&\textbf{Processing speed}\cr
\hline
44k&1m14.170s&598 bytes per second\cr
88k&2m25.540s&610 bytes per second\cr
176k&4m52.170s&608 bytes per second\cr
352k&9m42.200s&610 bytes per second\cr
704k&19m22.840s&611 bytes per second\cr
\hline
\end{tabular}
\end{center}
\caption{Experimental data: Times and speeds for complete runs on
         various input sizes with a fixed 300M heap}
\label{fig:times_speeds_fixed_heap}
\end{figure}

The parsing phase of {\HaskellToLaTeX} successfully pulls together a number of
existing technologies; the result is acceptably fast on modern machines.
We have also made it possible to stop between each stage and output
either the data structures as shown by the derived \htycon{Show} instances
or a simple pretty-printing function. Some sample input together with
some of the outputs possible are shown in Figure~\ref{fig:stages};
output is possible at a few additional points, but the
pretty-printed version is identical to one of the others.

\begin{figure}
\begin{obeyspaces}
\subfigure[Original input]{
\begin{tabular}{l}
\texttt{foo x = y + z}\cr
\texttt{    where y = 2 * x}\cr
\texttt{          z = x + 3}\cr
\end{tabular}
}

\subfigure[After indent marking]{
\begin{tabular}{l}
\texttt{\{1\}foo x = y + z}\cr
\texttt{    <5>where \{11\}y = 2 * x}\cr
\texttt{          <11>z = x + 3}\cr
\end{tabular}
}

\subfigure[After layout]{
\begin{tabular}{l}
\texttt{\{foo x = y + z}\cr
\texttt{    where \{y = 2 * x}\cr
\texttt{          ;z = x + 3}\cr
\texttt{\}\}}\cr
\end{tabular}
}

\subfigure[Parseable tokens]{
\begin{tabular}{l}     
\texttt{\{foox=y+zwhere\{y=2*x;z=x+3\}\}}\cr
\end{tabular}
}
\end{obeyspaces}

\setbox0\hbox{
\begin{code}
foo x = y + z
    where y = 2 * x
          z = x + 3
\end{code}
}

\subfigure[Code]{
\box0
}
\caption{Output from the pretty-printer at various stages}
\label{fig:stages}
\end{figure}

The {\LaTeX} output phase is more prototypical; although it produces
acceptable output for simple functions, more complex functions can lead
to poor layout. Many of the difficulties faced in the output phase are
due to limitations in {\LaTeX} and {\TeX}. They were designed to be used
to typeset English text and mathematics, while we require a more precise
and restrictive layout. However, there are still acceptable layouts for
many of the constructs, but to support more than one layout for a given
construct requires a lot of {\TeX} programming; a typesetting language
designed with this sort of layout in mind would provide an easier target
for a high quality Haskell pretty-printer. One problem with {\TeX} that
is very hard to get round is that anything surrounded by automatically
sized parentheses or square brackets, such as the case statement in the
function below, is never split across multiple lines no matter how wide
it is.

\begin{code}
foo x = (case x of
             0 -> "hello"
             _ -> "goodbye") ++ " world"
\end{code}

Another issue is that {\LaTeX} and {\TeX} allow hand tweaking by the
user as they accept that they will not be able to produce the optimum
result in all possible cases. This is harder to do in an automated
pretty-printer; it would be possible to pass hints and instructions to
the pretty-printer by means of Haskell comments, although this would
reduce the legibility of the plain source code.

\subsection{Other applications}

The Haskell parser we have developed is not limited to use in a {\LaTeX}
pretty-printer. Many other applications are conceivable, from merely
pretty-printing into different languages, e.g., HTML, to program
transformation systems like MAG~\cite{MAG}, or even a full Haskell
interpreter or compiler.

The parser combinator library is also reusable. With a \htycon{Position}
module which does not perform the unlitting, parsers for other languages
defined by syntax with similar semantics can be written. Indeed, as
talked about elsewhere in this report, this author has already written
two other parsers using the parser combinator library to aid in the
writing of the parser and of this report.

\subsection{Problems with Haskell report and implementations}

As well as the direct contribution to the Haskell community in the form
of a parser and pretty-printer for Haskell, several other contributions
have been made along the way. This project was written during a period
when the Haskell~98 report was being revised and many typos, errors and
inconsistencies were uncovered by this author.

During the implementation of the parser many bugs were found in each of
the three major implementations. Some of them were found when they
failed to compile or interpret the code for the parser correctly, some were
noticed when they did not accept the test-suite written to test the
parser and others were found when attempting to confirm the meaning of
the report by investigating what the implementations did and how they
differed from each other and what had been understood from the report.

% WORDCOUNT SECTION END

\clearpage
\renewcommand\refname{Bibliography}
\bibliographystyle{report}
\bibliography{report}

\clearpage
\appendix

\section{Automaton description}\label{app:automaton_diagrams}

These diagrams show the relationship between the lookup table of the
automaton and the Haskell context free grammar. The same program, which
is based on the PC library we have developed, is used to generate both
Lout~\cite{lout} mark-up files, which Lout then transforms into these
diagrams, and the executable code. Another program, also based on our PC
library, parses the EPS files created by Lout and removes unused
functions, comments and blank lines; this represents a space saving of
50\%, or approximately 5.5M on the uncompressed postscript document and
800k on the gzipped copy.

\vfill

\newcount\diag

\global\diag=0\relax
\loop\ifnum\diag<4\relax
\includegraphics[angle=90]{diag\the\diag}
\global\advance\diag by1\relax
\repeat

\begin{landscape}

\loop\ifnum\diag<70\relax
\noindent\includegraphics{diag\the\diag}

\global\advance\diag by1\relax
\repeat

\end{landscape}

\end{document}

